%*****************************************************************************80
%
%% Evaluates the objective function for problem 38.
%
%  Licensing:
%    This code is distributed under the GNU LGPL license.
%
%  Modified:
%    24 December 2011
%
%  Author:
%    Enrico Bertolazzi
%    Dipartimento di Ingegneria Meccanica e Strutturale
%    Universita` degli Studi di Trento
%    Via Mesiano 77, I-38123 Trento, Italy
%    email: enrico.bertolazzi@unitn.it
%
%    based on a code of John Burkardt (http://people.sc.fsu.edu/~jburkardt/)
%
%  Reference:
%
%    Zbigniew Michalewicz,
%    Genetic Algorithms + Data Structures = Evolution Programs,
%    Third Edition,
%    Springer Verlag, 1996,
%    ISBN: 3-540-60676-9,
%    LC: QA76.618.M53.
%
%    Richard Brent,
%    Algorithms for Minimization with Derivatives,
%    Dover, 2002,
%    ISBN: 0-486-41998-3,
%    LC: QA402.5.B74.
%
function RES = BohachevskyN2( what, x )
  global Nf_eval Ng_eval Nh_eval ;
  switch what

  case {'name','title'}
    RES = 'The Bohachevsky Function #2.' ;

  case {'n','size'}
    RES = 2 ;

  case 'm'
    RES = 1 ;

  case {'init','start'}
    RES = [ 0.6, 1.3 ]' ;

  case 'solution'
    RES = [ 0.0, 0.0 ]';

  case 'f'
    Nf_eval = Nf_eval+1 ;
    RES = eval_f( x ) ;

  case 'Df'
    Ng_eval = Ng_eval+1 ;
    RES = eval_g( x ) ;

  case 'F'
    Ng_eval = Ng_eval+1 ;
    RES = eval_g( x )' ;

  case {'JF', 'DDf'}
    Nh_eval = Nh_eval+1;
    RES = eval_h(x ) ;

  otherwise
    error(['The Bohachevsky Function #2, bad argument:' what]) ;
  end ;

end

%*****************************************************************************80
function f = eval_f ( x )
  f = x(1) * x(1) + 2.0 * x(2) * x(2) ...
    - 0.3 * cos ( 3.0 * pi * x(1) ) ...
    * cos ( 4.0 * pi * x(2) ) + 0.3;
end

%*****************************************************************************80
function g = eval_g ( x )
  g = zeros ( 1, 2 );

  g(1) = 2.0 * x(1) ...
    + 0.9 * pi * sin ( 3.0 * pi * x(1) ) ...
    * cos ( 4.0 * pi * x(2) );

  g(2) = 4.0 * x(2) ...
    + 1.2 * pi * cos ( 3.0 * pi * x(1) ) ...
    * sin ( 4.0 * pi * x(2) );

end

%*****************************************************************************80
function h = eval_h ( x )
  h = zeros ( 2, 2 );
  h(1,1) = 2.0 + 2.7 * pi^2 * cos ( 3.0 * pi * x(1) ) * cos ( 4.0 * pi * x(2) );
  h(1,2) = - 3.6 * pi^2 * sin ( 3.0 * pi * x(1) ) * sin ( 4.0 * pi * x(2) );
  h(2,1) = - 3.6 * pi^2 * sin ( 3.0 * pi * x(1) ) * sin ( 4.0 * pi * x(2) );
  h(2,2) = 4.0 + 4.8 * pi^2 * cos ( 3.0 * pi * x(1) ) * cos ( 4.0 * pi * x(2) );
end
