%*****************************************************************************80
%
%% Evaluates the objective function for problem 5.
%
%  Licensing:
%    This code is distributed under the GNU LGPL license.
%
%  Modified:
%    24 December 2011
%
%  Author:
%    Enrico Bertolazzi
%    Dipartimento di Ingegneria Meccanica e Strutturale
%    Universita` degli Studi di Trento
%    Via Mesiano 77, I-38123 Trento, Italy
%    email: enrico.bertolazzi@unitn.it
%
%    based on a code of John Burkardt (http://people.sc.fsu.edu/~jburkardt/)
%
%  Reference:
%    Richard Brent,
%    Algorithms for Minimization with Derivatives,
%    Dover, 2002,
%    ISBN: 0-486-41998-3,
%    LC: QA402.5.B74.
%
function RES = Box3(what,x)
  global Nf_eval Ng_eval Nh_eval ;
  switch what

  case {'name','title'}
    RES = 'The Box 3-dimensional function.' ;

  case {'n','size'}
    RES = 3 ;

  case 'm'
    RES = 1 ;

  case {'init','start'}
    RES = [ 0.0, 10.0, 5.0 ]' ;

  case 'solution'
    RES = [ 1.0, 10.0, 1.0 ]' ;

  case 'f'
    Nf_eval = Nf_eval+1 ;
    RES = eval_f( x ) ;

  case 'Df'
    Ng_eval = Ng_eval+1 ;
    RES = eval_g( x ) ;

  case 'F'
    Ng_eval = Ng_eval+1 ;
    RES = eval_g( x )' ;

  case {'JF', 'DDf'}
    Nh_eval = Nh_eval+1;
    RES = eval_h(x ) ;

  otherwise
    error(['The Box 3-dimensional function:' what]) ;
  end ;

end

%*****************************************************************************80
function f = eval_f ( x )
  f = 0.0;
  for i = 1 : 10
    c = - i / 10.0;
    fi = exp ( c * x(1) ) - exp ( c * x(2) ) - x(3) * ( exp ( c ) - exp ( 10.0 * c ) );
    f = f + fi * fi;
  end
end

%*****************************************************************************80
function g = eval_g ( x )
  g = zeros ( 1, 3 );

  for i = 1 : 10
    c = - i / 10.0;
    fi = exp ( c * x(1) ) - exp ( c * x(2) ) - x(3) * ( exp ( c ) - exp ( 10.0 * c ) );

    dfidx1 =   c * exp ( c * x(1) );
    dfidx2 = - c * exp ( c * x(2) );
    dfidx3 = - ( exp ( c ) - exp ( 10.0 * c ) );

    g(1) = g(1) + 2.0 * fi * dfidx1;
    g(2) = g(2) + 2.0 * fi * dfidx2;
    g(3) = g(3) + 2.0 * fi * dfidx3;

  end
end

%*****************************************************************************80
function h = eval_h ( x )
  h = zeros ( 3, 3 );

  for i = 1 : 10

    c = - i / 10.0;

    fi = exp ( c * x(1) ) - exp ( c * x(2) ) - x(3) * ( exp ( c ) - exp ( 10.0 * c ) );;

    dfidx1   =   c     * exp ( c * x(1) );
    d2fidx11 =   c * c * exp ( c * x(1) );
    dfidx2   = - c     * exp ( c * x(2) );
    d2fidx22 = - c * c * exp ( c * x(2) );
    dfidx3   = - ( exp ( c ) - exp ( 10.0 * c ) );

    h(1,1) = h(1,1) + 2.0 * dfidx1 * dfidx1 + 2.0 * fi * d2fidx11;
    h(1,2) = h(1,2) + 2.0 * dfidx1 * dfidx2;
    h(1,3) = h(1,3) + 2.0 * dfidx1 * dfidx3;

    h(2,1) = h(2,1) + 2.0 * dfidx2 * dfidx1;
    h(2,2) = h(2,2) + 2.0 * dfidx2 * dfidx2 + 2.0 * fi * d2fidx22;
    h(2,3) = h(2,3) + 2.0 * dfidx2 * dfidx3;

    h(3,1) = h(3,1) + 2.0 * dfidx3 * dfidx1;
    h(3,2) = h(3,2) + 2.0 * dfidx3 * dfidx2;
    h(3,3) = h(3,3) + 2.0 * dfidx3 * dfidx3;

  end

end
