%*****************************************************************************80
%
%% Evaluates the objective function for problem 30.
%
%  Licensing:
%    This code is distributed under the GNU LGPL license.
%
%  Modified:
%    24 December 2011
%
%  Author:
%    Enrico Bertolazzi
%    Dipartimento di Ingegneria Meccanica e Strutturale
%    Universita` degli Studi di Trento
%    Via Mesiano 77, I-38123 Trento, Italy
%    email: enrico.bertolazzi@unitn.it
%
%    based on a code of John Burkardt (http://people.sc.fsu.edu/~jburkardt/)
%
%  Reference:
%    Richard Brent,
%    Algorithms for Minimization with Derivatives,
%    Dover, 2002,
%    ISBN: 0-486-41998-3,
%    LC: QA402.5.B74.
%
function RES = BraninRCOS(what,x)
  global Nf_eval Ng_eval Nh_eval ;
  switch what

  case {'name','title'}
    RES = 'The Branin RCOS Function.' ;

  case {'n','size'}
    RES = 2 ;

  case 'm'
    RES = 1 ;

  case {'init','start'}
    RES = [ -1.0, 1.0 ]' ;

  case 'solution'
    RES = [ -pi, 12.275 ;
             pi,  2.275 ;
             9.42478, 2.475 ]';

  case 'f'
    Nf_eval = Nf_eval+1 ;
    RES = eval_f( x ) ;

  case 'Df'
    Ng_eval = Ng_eval+1 ;
    RES = eval_g( x ) ;

  case 'F'
    Ng_eval = Ng_eval+1 ;
    RES = eval_g( x )' ;

  case {'JF', 'DDf'}
    Nh_eval = Nh_eval+1;
    RES = eval_h(x ) ;

  otherwise
    error(['The Branin RCOS Function, bad argument:' what]) ;
  end ;

end

%*****************************************************************************80
function f = eval_f ( x )
  a = 1.0;
  d = 6.0;
  e = 10.0;

  b = 5.1 / ( 4.0 * pi^2 );
  c = 5.0 / pi;
  ff = 1.0 / ( 8.0 * pi );

  f = a * ( x(2) - b * x(1)^2 + c * x(1) - d )^2 + e * ( 1.0 - ff ) * cos ( x(1) ) + e;
end

%*****************************************************************************80
function g = eval_g ( x )
  g = zeros ( 1, 2 );

  a = 1.0;
  d = 6.0;
  e = 10.0;

  b = 5.1 / ( 4.0 * pi^2 );;
  c = 5.0 / pi;
  ff = 1.0 / ( 8.0 * pi );

  g(1) = 2.0 * a * ( x(2) - b * x(1)^2 + c * x(1) - d ) * ( - 2.0 * b * x(1) + c ) - e * ( 1.0 - ff ) * sin ( x(1) );
  g(2) = 2.0 * a * ( x(2) - b * x(1)^2 + c * x(1) - d ) ;
end

%*****************************************************************************80
function h = eval_h ( x )
  h = zeros ( 2, 2 ) ;

  a = 1.0;
  d = 6.0;
  e = 10.0;

  b  = 5.1 / ( 4.0 * pi^2 );
  c  = 5.0 / pi;
  ff = 1.0 / ( 8.0 * pi );

  h(1,1) = 2.0 * a * ( - 2.0 * b * x(1) + c ) ...
    * ( - 2.0 * b * x(1) + c ) ...
    - 4.0 * a * b * ( x(2) - b * x(1)^2 + c * x(1) - d ) ...
    - e * ( 1.0 - ff ) * cos ( x(1) );

  h(1,2) = 2.0 * a * ( - 2.0 * b * x(1) + c );
  h(2,1) = 2.0 * a * ( - 2.0 * b * x(1) + c );
  h(2,2) = 2.0 * a;

end
