%*****************************************************************************80
%
%% Evaluates the objective function for problem 24.
%
%  Licensing:
%    This code is distributed under the GNU LGPL license.
%
%  Modified:
%    24 December 2011
%
%  Author:
%    Enrico Bertolazzi
%    Dipartimento di Ingegneria Meccanica e Strutturale
%    Universita` degli Studi di Trento
%    Via Mesiano 77, I-38123 Trento, Italy
%    email: enrico.bertolazzi@unitn.it
%
%    based on a code of John Burkardt (http://people.sc.fsu.edu/~jburkardt/)
%
%  Reference:
%    Richard Brent,
%    Algorithms for Minimization with Derivatives,
%    Dover, 2002,
%    ISBN: 0-486-41998-3,
%    LC: QA402.5.B74.
%
function RES = DeJongF3(what,x)
  global Nf_eval Ng_eval Nh_eval ;
  switch what

  case {'name','title'}
    RES = 'The De Jong Function F3.' ;

  case {'n','size'}
    RES = 5 ;

  case 'm'
    RES = 1 ;

  case {'init','start'}
    RES = [ -5.12, -2.56, 0, 2.56, 5.12 ]' ;

  case 'solution'
    RES = [ -5, -5, -5, -5, -5 ]' ;

  case 'f'
    Nf_eval = Nf_eval+1 ;
    RES = eval_f( x ) ;

  case 'Df'
    Ng_eval = Ng_eval+1 ;
    RES = eval_g( x ) ;

  case 'F'
    Ng_eval = Ng_eval+1 ;
    RES = eval_g( x )' ;

  case {'JF', 'DDf'}
    Nh_eval = Nh_eval+1;
    RES = eval_h(x ) ;

  otherwise
    error(['DeJong function F3, bad argument:' what]) ;
  end ;

end

%*****************************************************************************80
function f = eval_f ( x )
  f = sum ( floor (x) ) ;
end

%*****************************************************************************80
function g = eval_g ( x )
  g = zeros ( 1, 5 ) ;
end

%*****************************************************************************80
function h = eval_h ( x )
  h = zeros ( 5, 5 ) ;
end
