%*****************************************************************************80
%
%% Evaluates the objective function for problem 26.
%
%  Licensing:
%    This code is distributed under the GNU LGPL license.
%
%  Modified:
%    24 December 2011
%
%  Author:
%    Enrico Bertolazzi
%    Dipartimento di Ingegneria Meccanica e Strutturale
%    Universita` degli Studi di Trento
%    Via Mesiano 77, I-38123 Trento, Italy
%    email: enrico.bertolazzi@unitn.it
%
%    based on a code of John Burkardt (http://people.sc.fsu.edu/~jburkardt/)
%
%  Reference:
%    Richard Brent,
%    Algorithms for Minimization with Derivatives,
%    Dover, 2002,
%    ISBN: 0-486-41998-3,
%    LC: QA402.5.B74.
%
function RES = DeJongF5(what,x)
  global Nf_eval Ng_eval Nh_eval ;
  switch what

  case {'name','title'}
    RES = 'The De Jong Function F5.' ;

  case {'n','size'}
    RES = 2 ;

  case 'm'
    RES = 1 ;

  case {'init','start'}
    RES = [ -32.01, -32.02 ]' ;

  case 'solution'
    RES = [ -32.0, -32.0 ]' ;

  case 'f'
    Nf_eval = Nf_eval+1 ;
    RES = eval_f( x ) ;

  case 'Df'
    Ng_eval = Ng_eval+1 ;
    RES = eval_g( x ) ;

  case 'F'
    Ng_eval = Ng_eval+1 ;
    RES = eval_g( x )' ;

  case {'JF', 'DDf'}
    Nh_eval = Nh_eval+1;
    RES = eval_h(x ) ;

  otherwise
    error(['DeJong function F5, bad argument:' what]) ;
  end ;

end

%*****************************************************************************80
function f = eval_f ( x )
  jroot = 5;
  k = 500.0;

  fi = k;

  for j = 1 : jroot * jroot

    j1 = mod ( j - 1, jroot ) + 1;
    a1 = -32 + j1 * 16;

    j2 = ( j - 1 ) / jroot;
    a2 = -32 + j2 * 16;

    fj = j + ( x(1) - a1 )^6 + ( x(2) - a2 )^6;

    fi = fi + 1.0 / fj;

  end

  f = 1.0 / fi;
end

%*****************************************************************************80
function g = eval_g ( X )
  g = zeros ( 1, 2 );
  x = X(1) ;
  y = X(2) ;
  % 
  % jroot  = 5   ;
  % k      = 500 ;
  % fi     = k   ;
  % dfidx1 = 0.0 ;
  % dfidx2 = 0.0 ;
  % 
  % for j = 1 : jroot * jroot
  % 
  %   j1 = mod ( j-1, jroot ) + 1;
  %   a1 = -32 + j1 * 16;
  % 
  %   j2 = ( j - 1 ) / jroot;
  %   a2 = -32 + j2 * 16;
  % 
  %   fj = ( j + ( x(1) - a1 )^6 + ( x(2) - a2 )^6 );
  % 
  %   fi = fi + 1.0 / fj;
  % 
  %   dfidx1 = dfidx1 - ( 1.0 / fj^2 ) * 6.0 * ( x(1) - a1 )^5;
  %   dfidx2 = dfidx2 - ( 1.0 / fj^2 ) * 6.0 * ( x(2) - a2 )^5;
  % 
  % end
  % 
  % g(1) = - ( 1.0 / fi^2 ) * dfidx1;
  % g(2) = - ( 1.0 / fi^2 ) * dfidx2;


	t1 = x - 32;
	t2 = t1 ^ 2;
	t3 = t2 ^ 2;
	t4 = t3 * t2;
	t5 = y + 0.32e2 / 0.5e1;
	t6 = t5 ^ 2;
	t7 = t6 ^ 2;
	t9 = 0.9e1 + t4 + t7 * t6;
	t12 = x - 16;
	t13 = t12 ^ 2;
	t14 = t13 ^ 2;
	t15 = t14 * t13;
	t16 = y + 0.48e2 / 0.5e1;
	t17 = t16 ^ 2;
	t18 = t17 ^ 2;
	t20 = 0.8e1 + t15 + t18 * t17;
	t23 = x - 48;
	t24 = t23 ^ 2;
	t25 = t24 ^ 2;
	t26 = t25 * t24;
	t27 = y - 0.224e3 / 0.5e1;
	t28 = t27 ^ 2;
	t29 = t28 ^ 2;
	t31 = 0.25e2 + t26 + t29 * t28;
	t34 = y - 0.208e3 / 0.5e1;
	t35 = t34 ^ 2;
	t36 = t35 ^ 2;
	t38 = 0.24e2 + t4 + t36 * t35;
	t41 = y - 0.192e3 / 0.5e1;
	t42 = t41 ^ 2;
	t43 = t42 ^ 2;
	t45 = 0.23e2 + t15 + t43 * t42;
	t48 = x ^ 2;
	t49 = t48 ^ 2;
	t50 = t49 * t48;
	t51 = y - 0.176e3 / 0.5e1;
	t52 = t51 ^ 2;
	t53 = t52 ^ 2;
	t55 = 0.22e2 + t50 + t53 * t52;
	t58 = y - 0.144e3 / 0.5e1;
	t59 = t58 ^ 2;
	t60 = t59 ^ 2;
	t62 = 0.20e2 + t26 + t60 * t59;
	t65 = x + 16;
	t66 = t65 ^ 2;
	t67 = t66 ^ 2;
	t68 = t67 * t66;
	t69 = y - 0.32e2;
	t70 = t69 ^ 2;
	t71 = t70 ^ 2;
	t73 = 0.21e2 + t68 + t71 * t70;
	t76 = y + 0.64e2 / 0.5e1;
	t77 = t76 ^ 2;
	t78 = t77 ^ 2;
	t80 = 0.7e1 + t50 + t78 * t77;
	t83 = y - 0.128e3 / 0.5e1;
	t84 = t83 ^ 2;
	t85 = t84 ^ 2;
	t87 = 0.19e2 + t4 + t85 * t84;
	t90 = y - 0.112e3 / 0.5e1;
	t91 = t90 ^ 2;
	t92 = t91 ^ 2;
	t94 = 0.18e2 + t15 + t92 * t91;
	t97 = y - 0.96e2 / 0.5e1;
	t98 = t97 ^ 2;
	t99 = t98 ^ 2;
	t101 = 0.17e2 + t50 + t99 * t98;
	t104 = 0.500e3 + 0.10e1 / t9 + 0.10e1 / t20 + 0.10e1 / t31 + 0.10e1 / t38 + 0.10e1 / t45 + 0.10e1 / t55 + 0.10e1 / t62 + 0.10e1 / t73 + 0.10e1 / t80 + 0.10e1 / t87 + 0.10e1 / t94 + 0.10e1 / t101;
	t105 = y - 0.16e2;
	t106 = t105 ^ 2;
	t107 = t106 ^ 2;
	t109 = 0.16e2 + t68 + t107 * t106;
	t112 = y - 0.64e2 / 0.5e1;
	t113 = t112 ^ 2;
	t114 = t113 ^ 2;
	t116 = 0.15e2 + t26 + t114 * t113;
	t119 = y - 0.48e2 / 0.5e1;
	t120 = t119 ^ 2;
	t121 = t120 ^ 2;
	t123 = 0.14e2 + t4 + t121 * t120;
	t126 = y - 0.32e2 / 0.5e1;
	t127 = t126 ^ 2;
	t128 = t127 ^ 2;
	t130 = 0.13e2 + t15 + t128 * t127;
	t133 = y - 0.16e2 / 0.5e1;
	t134 = t133 ^ 2;
	t135 = t134 ^ 2;
	t137 = 0.12e2 + t50 + t135 * t134;
	t140 = y + 0.16e2 / 0.5e1;
	t141 = t140 ^ 2;
	t142 = t141 ^ 2;
	t144 = 0.10e2 + t26 + t142 * t141;
	t147 = y ^ 2;
	t148 = t147 ^ 2;
	t150 = 0.11e2 + t68 + t148 * t147;
	t153 = y + 0.96e2 / 0.5e1;
	t154 = t153 ^ 2;
	t155 = t154 ^ 2;
	t157 = 0.5e1 + t26 + t155 * t154;
	t160 = y + 0.112e3 / 0.5e1;
	t161 = t160 ^ 2;
	t162 = t161 ^ 2;
	t164 = 0.4e1 + t4 + t162 * t161;
	t167 = y + 0.128e3 / 0.5e1;
	t168 = t167 ^ 2;
	t169 = t168 ^ 2;
	t171 = 0.3e1 + t15 + t169 * t168;
	t174 = y + 0.144e3 / 0.5e1;
	t175 = t174 ^ 2;
	t176 = t175 ^ 2;
	t178 = 0.2e1 + t50 + t176 * t175;
	t181 = y + 0.32e2;
	t182 = t181 ^ 2;
	t183 = t182 ^ 2;
	t185 = 0.1e1 + t68 + t183 * t182;
	t188 = y + 0.16e2;
	t189 = t188 ^ 2;
	t190 = t189 ^ 2;
	t192 = 0.6e1 + t68 + t190 * t189;
	t195 = 0.10e1 / t109 + 0.10e1 / t116 + 0.10e1 / t123 + 0.10e1 / t130 + 0.10e1 / t137 + 0.10e1 / t144 + 0.10e1 / t150 + 0.10e1 / t157 + 0.10e1 / t164 + 0.10e1 / t171 + 0.10e1 / t178 + 0.10e1 / t185 + 0.10e1 / t192;
	t197 = (t104 + t195) ^ 2;
	t198 = 0.1e1 / t197;
	t199 = t31 ^ 2;
	t200 = 0.1e1 / t199;
	t201 = t25 * t23;
	t204 = t20 ^ 2;
	t205 = 0.1e1 / t204;
	t206 = t14 * t12;
	t209 = t9 ^ 2;
	t210 = 0.1e1 / t209;
	t211 = t3 * t1;
	t214 = t38 ^ 2;
	t215 = 0.1e1 / t214;
	t218 = t55 ^ 2;
	t219 = 0.1e1 / t218;
	t220 = t49 * x;
	t223 = t45 ^ 2;
	t224 = 0.1e1 / t223;
	t227 = t62 ^ 2;
	t228 = 0.1e1 / t227;
	t231 = t94 ^ 2;
	t232 = 0.1e1 / t231;
	t235 = t87 ^ 2;
	t236 = 0.1e1 / t235;
	t239 = t80 ^ 2;
	t240 = 0.1e1 / t239;
	t243 = t73 ^ 2;
	t244 = 0.1e1 / t243;
	t245 = t67 * t65;
	t248 = t116 ^ 2;
	t249 = 0.1e1 / t248;
	t252 = -0.60e1 * t200 * t201 - 0.60e1 * t205 * t206 - 0.60e1 * t210 * t211 - 0.60e1 * t215 * t211 - 0.60e1 * t219 * t220 - 0.60e1 * t224 * t206 - 0.60e1 * t228 * t201 - 0.60e1 * t232 * t206 - 0.60e1 * t236 * t211 - 0.60e1 * t240 * t220 - 0.60e1 * t244 * t245 - 0.60e1 * t249 * t201;
	t253 = t109 ^ 2;
	t254 = 0.1e1 / t253;
	t257 = t101 ^ 2;
	t258 = 0.1e1 / t257;
	t261 = t123 ^ 2;
	t262 = 0.1e1 / t261;
	t265 = t130 ^ 2;
	t266 = 0.1e1 / t265;
	t269 = t137 ^ 2;
	t270 = 0.1e1 / t269;
	t273 = t144 ^ 2;
	t274 = 0.1e1 / t273;
	t277 = t150 ^ 2;
	t278 = 0.1e1 / t277;
	t281 = t192 ^ 2;
	t282 = 0.1e1 / t281;
	t285 = t185 ^ 2;
	t286 = 0.1e1 / t285;
	t289 = t178 ^ 2;
	t290 = 0.1e1 / t289;
	t293 = t171 ^ 2;
	t294 = 0.1e1 / t293;
	t297 = t164 ^ 2;
	t298 = 0.1e1 / t297;
	t301 = t157 ^ 2;
	t302 = 0.1e1 / t301;
	t305 = -0.60e1 * t254 * t245 - 0.60e1 * t258 * t220 - 0.60e1 * t262 * t211 - 0.60e1 * t266 * t206 - 0.60e1 * t270 * t220 - 0.60e1 * t274 * t201 - 0.60e1 * t278 * t245 - 0.60e1 * t282 * t245 - 0.60e1 * t286 * t245 - 0.60e1 * t290 * t220 - 0.60e1 * t294 * t206 - 0.60e1 * t298 * t211 - 0.60e1 * t302 * t201;
	t345 = -0.60e1 * t210 * t7 * t5 - 0.60e1 * t205 * t18 * t16 - 0.60e1 * t200 * t29 * t27 - 0.60e1 * t215 * t36 * t34 - 0.60e1 * t224 * t43 * t41 - 0.60e1 * t219 * t53 * t51 - 0.60e1 * t244 * t71 * t69 - 0.60e1 * t228 * t60 * t58 - 0.60e1 * t240 * t78 * t76 - 0.60e1 * t236 * t85 * t83 - 0.60e1 * t258 * t99 * t97 - 0.60e1 * t232 * t92 * t90;
	t385 = -0.60e1 * t249 * t114 * t112 - 0.60e1 * t254 * t107 * t105 - 0.60e1 * t262 * t121 * t119 - 0.60e1 * t270 * t135 * t133 - 0.60e1 * t266 * t128 * t126 - 0.60e1 * t302 * t155 * t153 - 0.60e1 * t278 * t148 * y - 0.60e1 * t274 * t142 * t140 - 0.60e1 * t298 * t162 * t160 - 0.60e1 * t294 * t169 * t167 - 0.60e1 * t290 * t176 * t174 - 0.60e1 * t282 * t190 * t188 - 0.60e1 * t286 * t183 * t181;
	g(1) = -0.10e1 * t198 * (t252 + t305);
	g(2) = -0.10e1 * t198 * (t345 + t385);
end

%*****************************************************************************80
function h = eval_h ( X )
  h = zeros ( 2, 2 ) ;
  x = X(1) ;
  y = X(2) ;
	t1 = x - 48;
	t2 = t1 ^ 2;
	t3 = t2 ^ 2;
	t4 = t3 * t2;
	t5 = y + 0.16e2 / 0.5e1;
	t6 = t5 ^ 2;
	t7 = t6 ^ 2;
	t9 = 0.10e2 + t4 + t7 * t6;
	t12 = y - 0.144e3 / 0.5e1;
	t13 = t12 ^ 2;
	t14 = t13 ^ 2;
	t16 = 0.20e2 + t4 + t14 * t13;
	t19 = x - 16;
	t20 = t19 ^ 2;
	t21 = t20 ^ 2;
	t22 = t21 * t20;
	t23 = y + 0.48e2 / 0.5e1;
	t24 = t23 ^ 2;
	t25 = t24 ^ 2;
	t27 = 0.8e1 + t22 + t25 * t24;
	t30 = x - 32;
	t31 = t30 ^ 2;
	t32 = t31 ^ 2;
	t33 = t32 * t31;
	t34 = y - 0.48e2 / 0.5e1;
	t35 = t34 ^ 2;
	t36 = t35 ^ 2;
	t38 = 0.14e2 + t33 + t36 * t35;
	t41 = y - 0.112e3 / 0.5e1;
	t42 = t41 ^ 2;
	t43 = t42 ^ 2;
	t45 = 0.18e2 + t22 + t43 * t42;
	t48 = y - 0.128e3 / 0.5e1;
	t49 = t48 ^ 2;
	t50 = t49 ^ 2;
	t52 = 0.19e2 + t33 + t50 * t49;
	t55 = x ^ 2;
	t56 = t55 ^ 2;
	t57 = t56 * t55;
	t58 = y - 0.16e2 / 0.5e1;
	t59 = t58 ^ 2;
	t60 = t59 ^ 2;
	t62 = 0.12e2 + t57 + t60 * t59;
	t65 = x + 16;
	t66 = t65 ^ 2;
	t67 = t66 ^ 2;
	t68 = t67 * t66;
	t69 = y ^ 2;
	t70 = t69 ^ 2;
	t72 = 0.11e2 + t68 + t70 * t69;
	t75 = y + 0.96e2 / 0.5e1;
	t76 = t75 ^ 2;
	t77 = t76 ^ 2;
	t79 = 0.5e1 + t4 + t77 * t76;
	t82 = y + 0.112e3 / 0.5e1;
	t83 = t82 ^ 2;
	t84 = t83 ^ 2;
	t86 = 0.4e1 + t33 + t84 * t83;
	t89 = y + 0.128e3 / 0.5e1;
	t90 = t89 ^ 2;
	t91 = t90 ^ 2;
	t93 = 0.3e1 + t22 + t91 * t90;
	t96 = y + 0.144e3 / 0.5e1;
	t97 = t96 ^ 2;
	t98 = t97 ^ 2;
	t100 = 0.2e1 + t57 + t98 * t97;
	t103 = 0.500e3 + 0.10e1 / t9 + 0.10e1 / t16 + 0.10e1 / t27 + 0.10e1 / t38 + 0.10e1 / t45 + 0.10e1 / t52 + 0.10e1 / t62 + 0.10e1 / t72 + 0.10e1 / t79 + 0.10e1 / t86 + 0.10e1 / t93 + 0.10e1 / t100;
	t104 = y + 0.32e2;
	t105 = t104 ^ 2;
	t106 = t105 ^ 2;
	t108 = 0.1e1 + t68 + t106 * t105;
	t111 = y + 0.16e2;
	t112 = t111 ^ 2;
	t113 = t112 ^ 2;
	t115 = 0.6e1 + t68 + t113 * t112;
	t118 = y - 0.224e3 / 0.5e1;
	t119 = t118 ^ 2;
	t120 = t119 ^ 2;
	t122 = 0.25e2 + t4 + t120 * t119;
	t125 = y - 0.32e2;
	t126 = t125 ^ 2;
	t127 = t126 ^ 2;
	t129 = 0.21e2 + t68 + t127 * t126;
	t132 = y + 0.32e2 / 0.5e1;
	t133 = t132 ^ 2;
	t134 = t133 ^ 2;
	t136 = 0.9e1 + t33 + t134 * t133;
	t139 = y - 0.192e3 / 0.5e1;
	t140 = t139 ^ 2;
	t141 = t140 ^ 2;
	t143 = 0.23e2 + t22 + t141 * t140;
	t146 = y - 0.176e3 / 0.5e1;
	t147 = t146 ^ 2;
	t148 = t147 ^ 2;
	t150 = 0.22e2 + t57 + t148 * t147;
	t153 = y - 0.208e3 / 0.5e1;
	t154 = t153 ^ 2;
	t155 = t154 ^ 2;
	t157 = 0.24e2 + t33 + t155 * t154;
	t160 = y + 0.64e2 / 0.5e1;
	t161 = t160 ^ 2;
	t162 = t161 ^ 2;
	t164 = 0.7e1 + t57 + t162 * t161;
	t167 = y - 0.96e2 / 0.5e1;
	t168 = t167 ^ 2;
	t169 = t168 ^ 2;
	t171 = 0.17e2 + t57 + t169 * t168;
	t174 = y - 0.32e2 / 0.5e1;
	t175 = t174 ^ 2;
	t176 = t175 ^ 2;
	t178 = 0.13e2 + t22 + t176 * t175;
	t181 = y - 0.16e2;
	t182 = t181 ^ 2;
	t183 = t182 ^ 2;
	t185 = 0.16e2 + t68 + t183 * t182;
	t188 = y - 0.64e2 / 0.5e1;
	t189 = t188 ^ 2;
	t190 = t189 ^ 2;
	t192 = 0.15e2 + t4 + t190 * t189;
	t195 = 0.10e1 / t108 + 0.10e1 / t115 + 0.10e1 / t122 + 0.10e1 / t129 + 0.10e1 / t136 + 0.10e1 / t143 + 0.10e1 / t150 + 0.10e1 / t157 + 0.10e1 / t164 + 0.10e1 / t171 + 0.10e1 / t178 + 0.10e1 / t185 + 0.10e1 / t192;
	t196 = t103 + t195;
	t197 = t196 ^ 2;
	t199 = 0.1e1 / t197 / t196;
	t200 = t27 ^ 2;
	t201 = 0.1e1 / t200;
	t202 = t21 * t19;
	t205 = t122 ^ 2;
	t206 = 0.1e1 / t205;
	t207 = t3 * t1;
	t210 = t143 ^ 2;
	t211 = 0.1e1 / t210;
	t214 = t150 ^ 2;
	t215 = 0.1e1 / t214;
	t216 = t56 * x;
	t219 = t157 ^ 2;
	t220 = 0.1e1 / t219;
	t221 = t32 * t30;
	t224 = t136 ^ 2;
	t225 = 0.1e1 / t224;
	t228 = t52 ^ 2;
	t229 = 0.1e1 / t228;
	t232 = t45 ^ 2;
	t233 = 0.1e1 / t232;
	t236 = t16 ^ 2;
	t237 = 0.1e1 / t236;
	t240 = t164 ^ 2;
	t241 = 0.1e1 / t240;
	t244 = t129 ^ 2;
	t245 = 0.1e1 / t244;
	t246 = t67 * t65;
	t249 = t62 ^ 2;
	t250 = 0.1e1 / t249;
	t253 = -0.60e1 * t201 * t202 - 0.60e1 * t206 * t207 - 0.60e1 * t211 * t202 - 0.60e1 * t215 * t216 - 0.60e1 * t220 * t221 - 0.60e1 * t225 * t221 - 0.60e1 * t229 * t221 - 0.60e1 * t233 * t202 - 0.60e1 * t237 * t207 - 0.60e1 * t241 * t216 - 0.60e1 * t245 * t246 - 0.60e1 * t250 * t216;
	t254 = t178 ^ 2;
	t255 = 0.1e1 / t254;
	t258 = t38 ^ 2;
	t259 = 0.1e1 / t258;
	t262 = t171 ^ 2;
	t263 = 0.1e1 / t262;
	t266 = t185 ^ 2;
	t267 = 0.1e1 / t266;
	t270 = t192 ^ 2;
	t271 = 0.1e1 / t270;
	t274 = t100 ^ 2;
	t275 = 0.1e1 / t274;
	t278 = t108 ^ 2;
	t279 = 0.1e1 / t278;
	t282 = t115 ^ 2;
	t283 = 0.1e1 / t282;
	t286 = t72 ^ 2;
	t287 = 0.1e1 / t286;
	t290 = t9 ^ 2;
	t291 = 0.1e1 / t290;
	t294 = t79 ^ 2;
	t295 = 0.1e1 / t294;
	t298 = t86 ^ 2;
	t299 = 0.1e1 / t298;
	t302 = t93 ^ 2;
	t303 = 0.1e1 / t302;
	t306 = -0.60e1 * t255 * t202 - 0.60e1 * t259 * t221 - 0.60e1 * t263 * t216 - 0.60e1 * t267 * t246 - 0.60e1 * t271 * t207 - 0.60e1 * t275 * t216 - 0.60e1 * t279 * t246 - 0.60e1 * t283 * t246 - 0.60e1 * t287 * t246 - 0.60e1 * t291 * t207 - 0.60e1 * t295 * t207 - 0.60e1 * t299 * t221 - 0.60e1 * t303 * t202;
	t307 = t253 + t306;
	t308 = t307 ^ 2;
	t311 = 0.1e1 / t197;
	t336 = -0.300e2 * t255 * t21 - 0.300e2 * t259 * t32 - 0.300e2 * t263 * t56 - 0.300e2 * t267 * t67 - 0.300e2 * t245 * t67 - 0.300e2 * t241 * t56 - 0.300e2 * t271 * t3 - 0.300e2 * t211 * t21 - 0.300e2 * t233 * t21 - 0.300e2 * t237 * t3 - 0.300e2 * t229 * t32 - 0.300e2 * t295 * t3;
	t344 = 0.1e1 / t294 / t79;
	t345 = t3 ^ 2;
	t346 = t345 * t2;
	t350 = 0.1e1 / t298 / t86;
	t351 = t32 ^ 2;
	t352 = t351 * t31;
	t356 = 0.1e1 / t302 / t93;
	t357 = t21 ^ 2;
	t358 = t357 * t20;
	t362 = 0.1e1 / t274 / t100;
	t363 = t56 ^ 2;
	t364 = t363 * t55;
	t368 = 0.1e1 / t278 / t108;
	t369 = t67 ^ 2;
	t370 = t369 * t66;
	t374 = 0.1e1 / t282 / t115;
	t378 = 0.1e1 / t286 / t72;
	t382 = 0.1e1 / t290 / t9;
	t386 = 0.1e1 / t249 / t62;
	t390 = 0.1e1 / t254 / t178;
	t393 = -0.300e2 * t303 * t21 - 0.300e2 * t299 * t32 - 0.300e2 * t275 * t56 + 0.720e2 * t344 * t346 + 0.720e2 * t350 * t352 + 0.720e2 * t356 * t358 + 0.720e2 * t362 * t364 + 0.720e2 * t368 * t370 + 0.720e2 * t374 * t370 + 0.720e2 * t378 * t370 + 0.720e2 * t382 * t346 + 0.720e2 * t386 * t364 + 0.720e2 * t390 * t358;
	t396 = 0.1e1 / t258 / t38;
	t400 = 0.1e1 / t262 / t171;
	t404 = 0.1e1 / t266 / t185;
	t408 = 0.1e1 / t270 / t192;
	t412 = 0.1e1 / t244 / t129;
	t416 = 0.1e1 / t240 / t164;
	t420 = 0.1e1 / t228 / t52;
	t424 = 0.1e1 / t232 / t45;
	t428 = 0.1e1 / t236 / t16;
	t432 = 0.1e1 / t210 / t143;
	t436 = 0.1e1 / t214 / t150;
	t440 = 0.1e1 / t219 / t157;
	t443 = 0.720e2 * t396 * t352 + 0.720e2 * t400 * t364 + 0.720e2 * t404 * t370 + 0.720e2 * t408 * t346 + 0.720e2 * t412 * t370 + 0.720e2 * t416 * t364 + 0.720e2 * t420 * t352 + 0.720e2 * t424 * t358 + 0.720e2 * t428 * t346 + 0.720e2 * t432 * t358 + 0.720e2 * t436 * t364 + 0.720e2 * t440 * t352;
	t445 = 0.1e1 / t224 / t136;
	t449 = 0.1e1 / t200 / t27;
	t453 = 0.1e1 / t205 / t122;
	t476 = 0.720e2 * t445 * t352 + 0.720e2 * t449 * t358 + 0.720e2 * t453 * t346 - 0.300e2 * t206 * t3 - 0.300e2 * t201 * t21 - 0.300e2 * t220 * t32 - 0.300e2 * t225 * t32 - 0.300e2 * t215 * t56 - 0.300e2 * t250 * t56 - 0.300e2 * t291 * t3 - 0.300e2 * t283 * t67 - 0.300e2 * t287 * t67 - 0.300e2 * t279 * t67;
	t483 = t106 * t104;
	t486 = t113 * t111;
	t489 = t98 * t96;
	t492 = t91 * t89;
	t495 = t84 * t82;
	t498 = t7 * t5;
	t501 = t70 * y;
	t504 = t77 * t75;
	t507 = t176 * t174;
	t510 = t60 * t58;
	t513 = t36 * t34;
	t516 = t183 * t181;
	t519 = -0.60e1 * t279 * t483 - 0.60e1 * t283 * t486 - 0.60e1 * t275 * t489 - 0.60e1 * t303 * t492 - 0.60e1 * t299 * t495 - 0.60e1 * t291 * t498 - 0.60e1 * t287 * t501 - 0.60e1 * t295 * t504 - 0.60e1 * t255 * t507 - 0.60e1 * t250 * t510 - 0.60e1 * t259 * t513 - 0.60e1 * t267 * t516;
	t520 = t190 * t188;
	t523 = t43 * t41;
	t526 = t169 * t167;
	t529 = t50 * t48;
	t532 = t162 * t160;
	t535 = t14 * t12;
	t538 = t127 * t125;
	t541 = t148 * t146;
	t544 = t141 * t139;
	t547 = t155 * t153;
	t550 = t120 * t118;
	t553 = t25 * t23;
	t556 = t134 * t132;
	t559 = -0.60e1 * t271 * t520 - 0.60e1 * t233 * t523 - 0.60e1 * t263 * t526 - 0.60e1 * t229 * t529 - 0.60e1 * t241 * t532 - 0.60e1 * t237 * t535 - 0.60e1 * t245 * t538 - 0.60e1 * t215 * t541 - 0.60e1 * t211 * t544 - 0.60e1 * t220 * t547 - 0.60e1 * t206 * t550 - 0.60e1 * t201 * t553 - 0.60e1 * t225 * t556;
	t560 = t519 + t559;
	t599 = 0.720e2 * t378 * t246 * t501 + 0.720e2 * t404 * t246 * t516 + 0.720e2 * t416 * t216 * t532 + 0.720e2 * t368 * t246 * t483 + 0.720e2 * t362 * t216 * t489 + 0.720e2 * t350 * t221 * t495 + 0.720e2 * t344 * t207 * t504 + 0.720e2 * t400 * t216 * t526 + 0.720e2 * t396 * t221 * t513 + 0.720e2 * t382 * t207 * t498 + 0.720e2 * t374 * t246 * t486 + 0.720e2 * t408 * t207 * t520;
	t639 = 0.720e2 * t424 * t202 * t523 + 0.720e2 * t420 * t221 * t529 + 0.720e2 * t412 * t246 * t538 + 0.720e2 * t453 * t207 * t550 + 0.720e2 * t445 * t221 * t556 + 0.720e2 * t440 * t221 * t547 + 0.720e2 * t436 * t216 * t541 + 0.720e2 * t432 * t202 * t544 + 0.720e2 * t428 * t207 * t535 + 0.720e2 * t386 * t216 * t510 + 0.720e2 * t390 * t202 * t507 + 0.720e2 * t356 * t202 * t492 + 0.720e2 * t449 * t202 * t553;
	t643 = 0.20e1 * t199 * t307 * t560 - 0.10e1 * t311 * (t599 + t639);
	t644 = t560 ^ 2;
	t647 = t120 ^ 2;
	t651 = t155 ^ 2;
	t655 = t141 ^ 2;
	t659 = t148 ^ 2;
	t663 = t14 ^ 2;
	t667 = t127 ^ 2;
	t671 = t162 ^ 2;
	t675 = t50 ^ 2;
	t679 = t169 ^ 2;
	t683 = t43 ^ 2;
	t687 = t190 ^ 2;
	t691 = t91 ^ 2;
	t695 = 0.720e2 * t453 * t647 * t119 + 0.720e2 * t440 * t651 * t154 + 0.720e2 * t432 * t655 * t140 + 0.720e2 * t436 * t659 * t147 + 0.720e2 * t428 * t663 * t13 + 0.720e2 * t412 * t667 * t126 + 0.720e2 * t416 * t671 * t161 + 0.720e2 * t420 * t675 * t49 + 0.720e2 * t400 * t679 * t168 + 0.720e2 * t424 * t683 * t42 + 0.720e2 * t408 * t687 * t189 + 0.720e2 * t356 * t691 * t90;
	t696 = t84 ^ 2;
	t700 = t7 ^ 2;
	t704 = t70 ^ 2;
	t708 = t77 ^ 2;
	t712 = t176 ^ 2;
	t716 = t60 ^ 2;
	t720 = t36 ^ 2;
	t724 = t183 ^ 2;
	t728 = t98 ^ 2;
	t732 = t113 ^ 2;
	t736 = t106 ^ 2;
	t744 = 0.720e2 * t350 * t696 * t83 + 0.720e2 * t382 * t700 * t6 + 0.720e2 * t378 * t704 * t69 + 0.720e2 * t344 * t708 * t76 + 0.720e2 * t390 * t712 * t175 + 0.720e2 * t386 * t716 * t59 + 0.720e2 * t396 * t720 * t35 + 0.720e2 * t404 * t724 * t182 + 0.720e2 * t362 * t728 * t97 + 0.720e2 * t374 * t732 * t112 + 0.720e2 * t368 * t736 * t105 - 0.300e2 * t250 * t60 - 0.300e2 * t259 * t36;
	t770 = -0.300e2 * t267 * t183 - 0.300e2 * t271 * t190 - 0.300e2 * t233 * t43 - 0.300e2 * t263 * t169 - 0.300e2 * t229 * t50 - 0.300e2 * t241 * t162 - 0.300e2 * t237 * t14 - 0.300e2 * t245 * t127 - 0.300e2 * t215 * t148 - 0.300e2 * t211 * t141 - 0.300e2 * t220 * t155 - 0.300e2 * t206 * t120;
	t791 = t25 ^ 2;
	t797 = t134 ^ 2;
	t801 = -0.300e2 * t201 * t25 - 0.300e2 * t225 * t134 - 0.300e2 * t255 * t176 - 0.300e2 * t295 * t77 - 0.300e2 * t287 * t70 - 0.300e2 * t291 * t7 - 0.300e2 * t299 * t84 - 0.300e2 * t303 * t91 - 0.300e2 * t275 * t98 - 0.300e2 * t283 * t113 + 0.720e2 * t449 * t791 * t24 - 0.300e2 * t279 * t106 + 0.720e2 * t445 * t797 * t133;
	h(1,1) = 0.20e1 * t199 * t308 - 0.10e1 * t311 * (t336 + t393 + t443 + t476);
	h(1,2) = t643;
	h(2,1) = t643;
	h(2,2) = 0.20e1 * t199 * t644 - 0.10e1 * t311 * (t695 + t744 + t770 + t801);
end
