%*****************************************************************************80
%
%% Evaluates the objective function for problem 37.
%
%  Licensing:
%    This code is distributed under the GNU LGPL license.
%
%  Modified:
%    24 December 2011
%
%  Author:
%    Enrico Bertolazzi
%    Dipartimento di Ingegneria Meccanica e Strutturale
%    Universita` degli Studi di Trento
%    Via Mesiano 77, I-38123 Trento, Italy
%    email: enrico.bertolazzi@unitn.it
%
%    based on a code of John Burkardt (http://people.sc.fsu.edu/~jburkardt/)
%
%  Reference:
%    Richard Brent,
%    Algorithms for Minimization with Derivatives,
%    Dover, 2002,
%    ISBN: 0-486-41998-3,
%    LC: QA402.5.B74.
%
function RES = Easom( what, x )
  global Nf_eval Ng_eval Nh_eval ;
  switch what

  case {'name','title'}
    RES = 'The Easom Function.' ;

  case {'n','size'}
    RES = 2 ;

  case 'm'
    RES = 1 ;

  case {'init','start'}
    RES = [ 0.5, 1.0 ]' ;

  case 'solution'
    RES = [ pi, pi ]';

  case 'f'
    Nf_eval = Nf_eval+1 ;
    RES = eval_f( x ) ;

  case 'Df'
    Ng_eval = Ng_eval+1 ;
    RES = eval_g( x ) ;

  case 'F'
    Ng_eval = Ng_eval+1 ;
    RES = eval_g( x )' ;

  case {'JF', 'DDf'}
    Nh_eval = Nh_eval+1;
    RES = eval_h(x ) ;

  otherwise
    error(['The Easom Function, bad argument:' what]) ;
  end ;

end

%*****************************************************************************80
function f = eval_f ( x )
  arg = - ( x(1) - pi )^2 - ( x(2) - pi )^2;
  f   = - cos ( x(1) ) * cos ( x(2) ) * exp ( arg );
end

%*****************************************************************************80
function g = eval_g ( x )
  g = zeros ( 1, 2 );

  arg = - ( x(1) - pi )^2 - ( x(2) - pi )^2;

  g(1) = ( sin ( x(1) ) * cos ( x(2) ) ...
       + 2.0 * cos ( x(1) ) * cos ( x(2) ) * ( x(1) - pi ) ) ...
       * exp ( arg );

  g(2) = ( cos ( x(1) ) * sin ( x(2) ) ...
       + 2.0 * cos ( x(1) ) * cos ( x(2) ) * ( x(2) - pi ) ) ...
       * exp ( arg );

end

%*****************************************************************************80
function h = eval_h ( x )
  h = zeros ( 2, 2 );
  arg = - ( x(1) - pi )^2 - ( x(2) - pi )^2;
  dargdx1 = - 2.0 * ( x(1) - pi );
  dargdx2 = - 2.0 * ( x(2) - pi );

  factor = cos ( x(2) ) * ( sin ( x(1) ) - cos ( x(1) ) * dargdx1 );
  dfdx1 = cos ( x(2) ) * ( cos ( x(1) ) + sin ( x(1) ) * dargdx1 + 2.0 * cos ( x(1) ) );
  dfdx2 = - sin ( x(2) ) * ( sin ( x(1) ) - cos ( x(1) ) * dargdx1 );

  h(1,1) = ( dfdx1 + factor * dargdx1 ) * exp ( arg );
  h(1,2) = ( dfdx2 + factor * dargdx2 ) * exp ( arg );

  factor = cos ( x(1) ) * ( sin ( x(2) ) - cos ( x(2) ) * dargdx2 );
  dfdx1 = - sin ( x(1) ) * ( sin ( x(2) ) - cos ( x(2) ) * dargdx2 );
  dfdx2 = cos ( x(1) ) * ( cos ( x(2) ) + sin ( x(2) ) * dargdx2 ...
    + 2.0 * cos ( x(2) ) );

  h(2,1) = ( dfdx1 + factor * dargdx1 ) * exp ( arg );
  h(2,2) = ( dfdx2 + factor * dargdx2 ) * exp ( arg );
end
