%*****************************************************************************80
%
%% Evaluates the objective function for problem 14.
%
%  Licensing:
%    This code is distributed under the GNU LGPL license.
%
%  Modified:
%    24 December 2011
%
%  Author:
%    Enrico Bertolazzi
%    Dipartimento di Ingegneria Meccanica e Strutturale
%    Universita` degli Studi di Trento
%    Via Mesiano 77, I-38123 Trento, Italy
%    email: enrico.bertolazzi@unitn.it
%
%    based on a code of John Burkardt (http://people.sc.fsu.edu/~jburkardt/)
%
%  Reference:
%
%    Howard Rosenbrock,
%    An Automatic Method for Finding the Greatest or Least Value of a Function,
%    Computer Journal,
%    Volume 3, 1960, pages 175-184.
%
%    Richard Brent,
%    Algorithms for Minimization with Derivatives,
%    Dover, 2002,
%    ISBN: 0-486-41998-3,
%    LC: QA402.5.B74.
%
function RES = ExtendedRosenbrock(what,x)
  global Nf_eval Ng_eval Nh_eval ExtendedRosenbrock_n ;

  if isempty(ExtendedRosenbrock_n)
    ExtendedRosenbrock_n = 2;
  end

  switch what

  case 'setup'
    if x < 1
      error(['ExtendedRosenbrock(' what ') argument bust be >= 1']) ;
    end
    ExtendedRosenbrock_n = x ;

  case {'name','title'}
    RES = 'The Extended Rosenbrock parabolic valley Function.' ;

  case {'n','size'}
    RES = ExtendedRosenbrock_n ;

  case 'm'
    RES = 1 ;

  case {'init','start'}
    RES = zeros ( ExtendedRosenbrock_n, 1 );
    for i = 1 : ExtendedRosenbrock_n
      if ( mod ( i, 2 ) == 1 )
        RES(i) = - 1.2;
      else
        RES(i) = 1.0;
      end
    end

  case 'solution'
    RES = ones(ExtendedRosenbrock_n,1)  ;

  case 'f'
    Nf_eval = Nf_eval+1 ;
    RES = eval_f( x ) ;

  case 'Df'
    Ng_eval = Ng_eval+1 ;
    RES = eval_g( x ) ;

  case 'F'
    Ng_eval = Ng_eval+1 ;
    RES = eval_g( x )' ;

  case {'JF', 'DDf'}
    Nh_eval = Nh_eval+1;
    RES = eval_h(x ) ;

  otherwise
    error(['The Extended Rosenbrock parabolic valley Function, bad argument:' what]) ;
  end ;

end

%*****************************************************************************80
function f = eval_f ( x )
  global ExtendedRosenbrock_n ;
  n = ExtendedRosenbrock_n ;
  f = 0.0;
  for j = 1 : n
    if ( mod ( j, 2 ) == 1 )
      f = f + ( 1.0 - x(j) )^2;
    else
      f = f + 100.0 * ( x(j) - x(j-1) * x(j-1) )^2;
    end
  end
end

%*****************************************************************************80
function g = eval_g ( x )
  global ExtendedRosenbrock_n ;
  n = ExtendedRosenbrock_n ;
  g = zeros ( 1, n );

  for j = 1 : n
    if ( mod ( j, 2 ) == 1 )
      g(j)   = - 2.0 * ( 1.0 - x(j) );
    else
      g(j)   = 200.0 * ( x(j) - x(j-1) * x(j-1) );
      g(j-1) = g(j-1) - 400.0 * x(j-1) * ( x(j) - x(j-1) * x(j-1) );
    end
  end
end

%*****************************************************************************80
function h = eval_h ( x )
  global ExtendedRosenbrock_n ;
  n = ExtendedRosenbrock_n ;
  h = zeros ( n, n );

  for j = 1 : n
    if ( mod ( j, 2 ) == 1 )
      h(j,j) = 2.0;
    else
      h(j,j)     = 200.0;
      h(j,j-1)   = - 400.0 * x(j-1);
      h(j-1,j)   = h(j-1,j) - 400.0 * x(j-1);
      h(j-1,j-1) = h(j-1,j-1) - 400.0 * ( x(j) - 3.0 * x(j-1) * x(j-1) );
    end
  end
end
