%*****************************************************************************80
%
%% Evaluates the objective function for problem 3.
%
%  Licensing:
%    This code is distributed under the GNU LGPL license.
%
%  Modified:
%    24 December 2011
%
%  Author:
%    Enrico Bertolazzi
%    Dipartimento di Ingegneria Meccanica e Strutturale
%    Universita` degli Studi di Trento
%    Via Mesiano 77, I-38123 Trento, Italy
%    email: enrico.bertolazzi@unitn.it
%
%    based on a code of John Burkardt (http://people.sc.fsu.edu/~jburkardt/)
%
%  Reference:
%    Richard Brent,
%    Algorithms for Minimization with Derivatives,
%    Dover, 2002,
%    ISBN: 0-486-41998-3,
%    LC: QA402.5.B74.
%
function RES = Gauss(what,x)
  global Nf_eval Ng_eval Nh_eval ;
  switch what

  case {'name','title'}
    RES = 'The Gaussian function.' ;

  case {'n','size'}
    RES = 3 ;

  case 'm'
    RES = 1 ;

  case {'init','start'}
    RES = [ 0.4, 1.0, 0.0 ]' ;

  case 'solution'
    RES = [ 0.0, 0.0, 0.0 ]' ;

  case 'f'
    Nf_eval = Nf_eval+1 ;
    RES = eval_f( x ) ;

  case 'Df'
    Ng_eval = Ng_eval+1 ;
    RES = eval_g( x ) ;

  case 'F'
    Ng_eval = Ng_eval+1 ;
    RES = eval_g( x )' ;

  case {'JF', 'DDf'}
    Nh_eval = Nh_eval+1;
    RES = eval_h(x ) ;

  otherwise
    error(['The Gaussian function bad argument:' what]) ;
  end ;

end

%*****************************************************************************80
function y = yvec()

  y = [ 0.0009, 0.0044, 0.0175, 0.0540, ...
        0.1295, 0.2420, 0.3521, 0.3989, ...
        0.3521, 0.2420, 0.1295, 0.0540, ...
        0.0175, 0.0044, 0.0009 ] ;
end

%*****************************************************************************80
function f = eval_f ( x )
  y = yvec();
  f = 0.0;
  for i = 1 : 15
    t = x(1) * exp ( - 0.5 * x(2) * ( 3.5 - 0.5 * ( i - 1 ) - x(3) )^2 ) - y(i);
    f = f + t * t;
  end
end

%*****************************************************************************80
function g = eval_g ( x )
  y = yvec();
  g = zeros ( 1, 3 );
  for i = 1 : 15
    d1  = 0.5 * ( i - 1 );
    d2  = 3.5 - d1 - x(3);
    arg = - 0.5 * x(2) * d2 * d2;
    t   = x(1) * exp ( arg ) - y(i);

    g(1) = g(1) + 2.0 * exp ( arg ) * t;
    g(2) = g(2) - x(1) * exp ( arg ) * t * d2 * d2;
    g(3) = g(3) + 2.0 * x(1) * x(2) * exp ( arg ) * t * d2;
  end
end

%*****************************************************************************80
function h = eval_h ( x )
  y = yvec();
  h = zeros ( 3, 3 );

  for i = 1 : 15

    d1 = 0.5 * ( i - 1 );
    d2 = 3.5 - d1 - x(3);
    arg = 0.5 * x(2) * d2 * d2;
    r = exp ( - arg );
    t = x(1) * r - y(i);
    t1 = 2.0 * x(1) * r - y(i);

    h(1,1) = h(1,1) + r * r;
    h(2,2) = h(2,2) + r * t1 * d2^4;
    h(3,3) = h(3,3) + r * ( x(2) * t1 * d2 * d2 - t );
    h(2,1) = h(2,1) - r * t1 * d2 * d2;
    h(3,1) = h(3,1) + d2 * r * t1;
    h(3,2) = h(3,2) + d2 * r * ( t - arg * t1 );

  end

  h(1,1) = 2.0 * h(1,1);
  h(2,2) = 0.5 * x(1) * h(2,2);
  h(3,3) = 2.0 * x(1) * x(2) * h(3,3);
  h(3,1) = 2.0 * x(2) * h(3,1);
  h(3,2) = 2.0 * x(1) * h(3,2);

  h(1,2) = h(2,1);
  h(1,3) = h(3,1);
  h(2,3) = h(3,2);
end
