%*****************************************************************************80
%
%% Evaluates the objective function for problem 6.
%
%  Licensing:
%    This code is distributed under the GNU LGPL license.
%
%  Modified:
%    24 December 2011
%
%  Author:
%    Enrico Bertolazzi
%    Dipartimento di Ingegneria Meccanica e Strutturale
%    Universita` degli Studi di Trento
%    Via Mesiano 77, I-38123 Trento, Italy
%    email: enrico.bertolazzi@unitn.it
%
%    based on a code of John Burkardt (http://people.sc.fsu.edu/~jburkardt/)
%
%  Reference:
%    Richard Brent,
%    Algorithms for Minimization with Derivatives,
%    Dover, 2002,
%    ISBN: 0-486-41998-3,
%    LC: QA402.5.B74.
%
function RES = Hilbert( what, x )
  global Nf_eval Ng_eval Nh_eval Hilbert_n ;

  if isempty(Hilbert_n)
    Hilbert_n = 2;
  end

  switch what

  case 'setup'
    if x < 1 
      error(['Hilbert(''setup'',' x ') argument must be >= 1']) ; 
    end
    Hilbert_n = x ;

  case {'name','title'}
    RES = 'The Hilbert Matrix Function F = x''Ax.' ;

  case {'n','size'}
    RES = Hilbert_n ;

  case 'm'
    RES = 1 ;

  case {'init','start'}
    RES = ones ( Hilbert_n, 1 );

  case 'solution'
    RES = zeros( Hilbert_n, 1 ) ;

  case 'f'
    Nf_eval = Nf_eval+1 ;
    RES = eval_f( x ) ;

  case 'Df'
    Ng_eval = Ng_eval+1 ;
    RES = eval_g( x ) ;

  case 'F'
    Ng_eval = Ng_eval+1 ;
    RES = eval_g( x )' ;

  case {'JF', 'DDf'}
    Nh_eval = Nh_eval+1;
    RES = eval_h(x ) ;

  otherwise
    error(['The Hilbert Matrix Function F = x''Ax, bad argument:' what]) ;
  end ;

end

%*****************************************************************************80
function f = eval_f ( x )
  global Hilbert_n ;
  n = Hilbert_n ;
  f = 0.0;
  for i = 1 : n
    for j = 1 : n
      f = f + x(i) * x(j) / ( i + j - 1 );
    end
  end
end

%*****************************************************************************80
function g = eval_g ( x )
  global Hilbert_n ;
  n = Hilbert_n ;
  g = zeros ( 1, n );
  for i = 1 : n
    g(i) = 0.0;
    for j = 1 : n
      g(i) = g(i) + 2.0 * x(j) / ( i + j - 1 );
    end
  end
end

%*****************************************************************************80
function h = eval_h ( x )
  global Hilbert_n ;
  n = Hilbert_n ;
  h = zeros ( n, n );
  for i = 1 : n
    for j = 1 : n
      h(i,j) = 2.0 / ( i + j - 1 );
    end
  end 
end
