%*****************************************************************************80
%
%% Evaluates the objective function for problem 19.
%
%  Licensing:
%    This code is distributed under the GNU LGPL license.
%
%  Modified:
%    24 December 2011
%
%  Author:
%    Enrico Bertolazzi
%    Dipartimento di Ingegneria Meccanica e Strutturale
%    Universita` degli Studi di Trento
%    Via Mesiano 77, I-38123 Trento, Italy
%    email: enrico.bertolazzi@unitn.it
%
%    based on a code of John Burkardt (http://people.sc.fsu.edu/~jburkardt/)
%
%  Reference:
%    Richard Brent,
%    Algorithms for Minimization with Derivatives,
%    Dover, 2002,
%    ISBN: 0-486-41998-3,
%    LC: QA402.5.B74.
%
function RES = Leon( what, x )

  global Nf_eval Ng_eval Nh_eval ;
  switch what

  case {'name','title'}
    RES = 'The Leon cubic valley function.' ;

  case {'n','size'}
    RES = 2 ;

  case 'm'
    RES = 1 ;

  case {'init','start'}
    RES = [ -1.2, -1.0 ]' ;

  case 'solution'
    RES = [ 1.0, 1.0 ]' ;

  case 'f'
    Nf_eval = Nf_eval+1 ;
    RES = eval_f( x ) ;

  case 'Df'
    Ng_eval = Ng_eval+1 ;
    RES = eval_g( x ) ;

  case 'F'
    Ng_eval = Ng_eval+1 ;
    RES = eval_g( x )' ;

  case {'JF', 'DDf'}
    Nh_eval = Nh_eval+1;
    RES = eval_h( x ) ;

  otherwise
    error 'The Leon cubic valley function, bad argument: %1', what ;
  end ;

end

%*****************************************************************************80
function f = eval_f ( x )
  f1 = x(2) - x(1) * x(1) * x(1);
  f2 = 1.0 - x(1);
  f  = 100.0 * f1 * f1 + f2 * f2;
end

%*****************************************************************************80
function g = eval_g ( x )
  g    = zeros ( 1, 2 );
  g(1) = - 600.0 * ( x(2) - x(1)^3 ) * x(1) * x(1) - 2.0 * ( 1.0 - x(1) );
  g(2) = 200.0 * ( x(2) - x(1)^3 );
end

%*****************************************************************************80
function h = eval_h ( x )
  h = zeros ( 2, 2 );
  h(1,1) = - 1200.0 * x(1) * x(2) + 3000.0 * x(1)^4 + 2.0;
  h(1,2) = - 600.0 * x(1) * x(1);
  h(2,1) = - 600.0 * x(1) * x(1);
  h(2,2) = 200.0;
end
