%*****************************************************************************80
%
%% Evaluates the objective function for problem 8.
%
%  Licensing:
%    This code is distributed under the GNU LGPL license.
%
%  Modified:
%    24 December 2011
%
%  Author:
%    Enrico Bertolazzi
%    Dipartimento di Ingegneria Meccanica e Strutturale
%    Universita` degli Studi di Trento
%    Via Mesiano 77, I-38123 Trento, Italy
%    email: enrico.bertolazzi@unitn.it
%
%    based on a code of John Burkardt (http://people.sc.fsu.edu/~jburkardt/)
%
%  Reference:
%    Richard Brent,
%    Algorithms for Minimization with Derivatives,
%    Dover, 2002,
%    ISBN: 0-486-41998-3,
%    LC: QA402.5.B74.
%
function RES = PenaltyN1(what,x)
  global Nf_eval Ng_eval Nh_eval PenaltyN1_n ;

  if isempty(PenaltyN1_n)
    PenaltyN1_n = 2;
  end

  switch what

  case 'setup'
    if x < 2
      error(['PenaltyN1(''setup'', ' x ') argument must be >= 2']) ; 
    end
    Watson_n = x ;

  case {'name','title'}
    RES = 'The Penalty Function #1.' ;

  case {'n','size'}
    RES = PenaltyN1_n ;

  case 'm'
    RES = 1 ;

  case {'init','start'}
    RES = (1:PenaltyN1_n)';

  case 'solution'
    RES = [] ; % soluzione non nota

  case 'f'
    Nf_eval = Nf_eval+1 ;
    RES = eval_f( x ) ;

  case 'Df'
    Ng_eval = Ng_eval+1 ;
    RES = eval_g( x ) ;

  case 'F'
    Ng_eval = Ng_eval+1 ;
    RES = eval_g( x )' ;

  case {'JF', 'DDf'}
    Nh_eval = Nh_eval+1;
    RES = eval_h(x ) ;

  otherwise
    error(['The Penalty Function #1, bad argument:' what]) ;
  end ;

end

%*****************************************************************************80
function f = eval_f ( x )
  global PenaltyN1_n ;
  n  = PenaltyN1_n ;
  ap = 0.00001;
  t1 = - 0.25 + dot( x, x ) ;
  t2 = sum ( ( x - 1.0 ).^2 );
  f  = ap * t2 + t1 * t1;
end

%*****************************************************************************80
function g = eval_g ( x )
  global PenaltyN1_n ;
  n  = PenaltyN1_n ;
  ap = 0.00001;
  t1 = - 0.25 + dot( x, x ) ;
  g(1,1:n) = (2.0 * ap) * ( x' - 1.0 ) + (4.0 * t1) * x' ;
end

%*****************************************************************************80
function h = eval_h ( x )
  global PenaltyN1_n ;
  n = PenaltyN1_n ;
  ap = 0.00001;

  t1 = - 0.25 + dot( x, x );

  d1 = 2.0 * ap;
  th = 4.0 * t1;

  for i = 1 : n
    h(i,i) = d1 + th + 8.0 * x(i)^2;
    for j = 1 : i - 1
      h(i,j) = 8.0 * x(i) * x(j);
    end
  end

  for i = 1 : n
    for j = i + 1 : n
      h(i,j) = h(j,i);
    end
  end

end
