%*****************************************************************************80
%
%% Evaluates the objective function for problem 9.
%
%  Licensing:
%    This code is distributed under the GNU LGPL license.
%
%  Modified:
%    24 December 2011
%
%  Author:
%    Enrico Bertolazzi
%    Dipartimento di Ingegneria Meccanica e Strutturale
%    Universita` degli Studi di Trento
%    Via Mesiano 77, I-38123 Trento, Italy
%    email: enrico.bertolazzi@unitn.it
%
%    based on a code of John Burkardt (http://people.sc.fsu.edu/~jburkardt/)
%
%  Reference:
%    Richard Brent,
%    Algorithms for Minimization with Derivatives,
%    Dover, 2002,
%    ISBN: 0-486-41998-3,
%    LC: QA402.5.B74.
%
function RES = PenaltyN2(what,x)
  global Nf_eval Ng_eval Nh_eval PenaltyN2_n ;

  if isempty(PenaltyN2_n)
    PenaltyN2_n = 2;
  end

  switch what

  case 'setup'
    if x < 1
      error(['PenaltyN2(''setup'', ' x ') argument must be >= 1']) ; 
    end
    PenaltyN2_n = x ;

  case {'name','title'}
    RES = 'The Penalty Function #2.' ;

  case {'n','size'}
    RES = PenaltyN2_n ;

  case 'm'
    RES = 1 ;

  case {'init','start'}
    RES = 0.5 * ones ( PenaltyN2_n, 1 ) ;

  case 'solution'
    RES = [] ; % soluzione non nota

  case 'f'
    Nf_eval = Nf_eval+1 ;
    RES = eval_f( x ) ;

  case 'Df'
    Ng_eval = Ng_eval+1 ;
    RES = eval_g( x ) ;

  case 'F'
    Ng_eval = Ng_eval+1 ;
    RES = eval_g( x )' ;

  case {'JF', 'DDf'}
    Nh_eval = Nh_eval+1;
    RES = eval_h(x ) ;

  otherwise
    error(['The Penalty Function #2, bad argument:' what]) ;
  end ;

end

%*****************************************************************************80
function f = eval_f ( x )
  global PenaltyN2_n ;
  n  = PenaltyN2_n ;
  ap = 0.00001;

  t1 = -1.0;
  t2 = 0.0;
  t3 = 0.0;
  d2 = 1.0;
  s2 = 0.0;

  for j = 1 : n
    t1 = t1 + ( n - j + 1 ) * x(j)^2;
    s1 = exp ( x(j) / 10.0 );
    if ( 1 < j )
      s3 = s1 + s2 - d2 * ( exp ( 0.1 ) + 1.0 );
      t2 = t2 + s3 * s3;
      t3 = t3 + ( s1 - 1.0 / exp ( 0.1 ) )^2;
    end
    s2 = s1;
    d2 = d2 * exp ( 0.1 );
  end

  f = ap * ( t2 + t3 ) + t1 * t1 + ( x(1) - 0.2 )^2;
end

%*****************************************************************************80
function g = eval_g ( x )
  global PenaltyN2_n ;
  n  = PenaltyN2_n ;
  g = zeros ( 1, n );

  ap = 0.00001;

  t1 = -1.0;
  for j = 1 : n
    t1 = t1 + ( n - j + 1 ) * x(j)^2;
  end

  d2 = 1.0;
  th = 4.0 * t1;
  s2 = 0.0;
  for j = 1 : n
    g(j) = ( n - j + 1 ) * x(j) * th;
    s1 = exp ( x(j) / 10.0 );
    if ( 1 < j )
      s3 = s1 + s2 - d2 * ( exp ( 0.1 ) + 1.0 );
      g(j) = g(j) + ap * s1 * ( s3 + s1 - 1.0 / exp ( 0.1 ) ) / 5.0;
      g(j-1) = g(j-1) + ap * s2 * s3 / 5.0;
    end
    s2 = s1;
    d2 = d2 * exp ( 0.1 );
  end

  g(1) = g(1) + 2.0 * ( x(1) - 0.2 );
end

%*****************************************************************************80
function h = eval_h ( x )
  global PenaltyN2_n ;
  n = PenaltyN2_n ;
  ap = 0.00001;

  t1 = - 1.0;
  for j = 1 : n
    t1 = t1 + ( n - j + 1 ) * x(j) * x(j);
  end

  d1 = exp ( 0.1 );
  d2 = 1.0;
  s2 = 0.0;
  th = 4.0 * t1;

  for j = 1 : n

    h(j,j) = 8.0 * ( ( n - j + 1 ) * x(j) )^2 + ( n - j + 1 ) * th;

    s1 = exp ( x(j) / 10.0 );

    if ( 1 < j )

      s3 = s1 + s2 - d2 * ( d1 + 1.0 );
      h(j,j) = h(j,j) + ap * s1 * ( s3 + s1 - 1.0 ...
        / d1 + 2.0 * s1 ) / 50.0;
      h(j-1,j-1) = h(j-1,j-1) + ap * s2 * ( s2 + s3 ) / 50.0;
      for k = 1 : j - 1
        t1 = exp ( k / 10.0 );
        h(j,k) = 8.0 * ( n - j + 1 ) *  ( n - k + 1 ) * x(j) * x(k);
      end

      h(j,j-1) = h(j,j-1) + ap * s1 * s2 / 50.0;

    end

    s2 = s1;
    d2 = d1 * d2;

  end

  h(1,1) = h(1,1) + 2.0;

  for i = 1 : n
    h(i,i+1:n) = h(i+1:n,i);
  end

end
