%*****************************************************************************80
%
%  Licensing:
%    This code is distributed under the GNU LGPL license.
%
%  Modified:
%    24 December 2011
%
%  Author:
%    Enrico Bertolazzi
%    Dipartimento di Ingegneria Meccanica e Strutturale
%    Universita` degli Studi di Trento
%    Via Mesiano 77, I-38123 Trento, Italy
%    email: enrico.bertolazzi@unitn.it
%
%    based on a code of John Burkardt (http://people.sc.fsu.edu/~jburkardt/)
%
%  Reference:
%
%    Michael Powell,
%    An Iterative Method for Finding Stationary Values of a Function
%    of Several Variables,
%    Computer Journal,
%    Volume 5, 1962, pages 147-151.
%
function RES = PowellSingularQuartic( what, x )
  global Nf_eval Ng_eval Nh_eval ;
  switch what

  case {'name','title'}
    RES = 'The Powell singular quartic function.' ;

  case {'n','size'}
    RES = 4 ;

  case 'm'
    RES = 1 ;

  case {'init','start'}
    RES = [ 3.0, -1.0, 0.0, 1.0 ]' ;

  case 'solution'
    RES = zeros(4,1) ;

  case 'f'
    Nf_eval = Nf_eval+1 ;
    RES = eval_f( x ) ;

  case 'Df'
    Ng_eval = Ng_eval+1 ;
    RES = eval_g( x ) ;

  case 'F'
    Ng_eval = Ng_eval+1 ;
    RES = eval_g( x )' ;

  case {'JF', 'DDf'}
    Nh_eval = Nh_eval+1;
    RES = eval_h(x ) ;

  otherwise
    error(['The Powell singular quartic function, bad argument:' what]) ;
  end ;

end

%*****************************************************************************80
function f = eval_f ( x )
  f1 = x(1) + 10.0 * x(2);
  f2 =                            x(3) - x(4);
  f3 =               x(2) - 2.0 * x(3);
  f4 = x(1)                            - x(4);
  f = f1 * f1 + f2 * f2 + f3 * f3 + f4 * f4;
end

%*****************************************************************************80
function g = eval_g ( x )
  g = [4 * x(1) + 20  * x(2) - 2 * x(4), ...
      20 * x(1) + 202 * x(2) - 4 * x(3), ...
      10 * x(3) - 2   * x(4) - 4 * x(2), ...
      -2 * x(3) + 4   * x(4) - 2 * x(1) ] ;
end

%*****************************************************************************80
function h = eval_h ( X )
  h = [4 20 0 -2; 20 202 -4 0; 0 -4 10 -2; -2 0 -2 4;];
end
