%
% Rosenbrock test
%
% what = 'f'  compute the Rosenbrock function
% what = 'Df' compute the Gradient of the Rosenbrock function
% what = 'DDf' compute the Hessian of the Rosenbrock function
%
%  Reference:
%
%    Howard Rosenbrock,
%    An Automatic Method for Finding the Greatest or Least Value of a Function,
%    Computer Journal,
%    Volume 3, 1960, pages 175-184.
%
function RES = Rosenbrock( what, XIN )
  global Nf_eval Ng_eval Nh_eval ;

  if nargin > 1
    X  = XIN(1,:) ;
    Y  = XIN(2,:) ;
    YX = Y - X.^2 ;
  end

  switch what

  case {'name','title'}
    RES = 'Rosenbrock' ;

  case { 'n', 'size'}
    RES = 2 ;

  case 'm'
    RES = 1 ;

  case {'init','start'}
    RES = [ -1.2 ; 1 ] ;

  case 'solution'
    RES = [ 1 ; 1 ] ;

  case 'f'
    Nf_eval = Nf_eval+1 ;
    RES = 100*YX.^2+(X-1).^2 ;

  case 'Df'
    Ng_eval = Ng_eval+1 ;
    RES = [ 2*(X-1) - 400*YX.*X, 200*YX ] ;

  case 'F'
    Ng_eval = Ng_eval+1 ;
    RES = [ 2*(X-1) - 400*YX.*X, 200*YX ]' ;

  case {'JF', 'DDf'}
    Nh_eval = Nh_eval+1;
    RES = [ 1200 * X.^2 - 400 *Y + 2, -400*X ; ...
            -400*X,                    200*ones(size(X)) ] ;
  otherwise
    error( [ 'Rosenbrock bad argument: ' what] ) ;
  end ;
end