%*****************************************************************************80
%
%% Evaluates the objective function for problem 27.
%
%  Licensing:
%    This code is distributed under the GNU LGPL license.
%
%  Modified:
%    24 December 2011
%
%  Author:
%    Enrico Bertolazzi
%    Dipartimento di Ingegneria Meccanica e Strutturale
%    Universita` degli Studi di Trento
%    Via Mesiano 77, I-38123 Trento, Italy
%    email: enrico.bertolazzi@unitn.it
%
%    based on a code of John Burkardt (http://people.sc.fsu.edu/~jburkardt/)
%
%  Reference:
%    Richard Brent,
%    Algorithms for Minimization with Derivatives,
%    Dover, 2002,
%    ISBN: 0-486-41998-3,
%    LC: QA402.5.B74.
%
function RES = SchafferF7(what,x)
  global Nf_eval Ng_eval Nh_eval ;
  switch what

  case {'name','title'}
    RES = 'The Schaffer Function F7.' ;

  case {'n','size'}
    RES = 2 ;

  case 'm'
    RES = 1 ;

  case {'init','start'}
    RES = [-5.0, +10.0]' ;

  case 'solution'
    RES = zeros( 2, 1 ) ;

  case 'f'
    Nf_eval = Nf_eval+1 ;
    RES = eval_f( x ) ;

  case 'Df'
    Ng_eval = Ng_eval+1 ;
    RES = eval_g( x ) ;

  case 'F'
    Ng_eval = Ng_eval+1 ;
    RES = eval_g( x )' ;

  case {'JF', 'DDf'}
    Nh_eval = Nh_eval+1;
    RES = eval_h(x ) ;

  otherwise
    error(['SchafferF7 function, bad argument:' what]) ;
  end ;

end

%*****************************************************************************80
function f = eval_f ( x )
  r = sqrt ( x(1)^2 + x(2)^2 );
  f = sqrt ( r ) * ( 1.0 + ( sin ( 50.0 * r^0.2 ) )^2 );
end

%*****************************************************************************80
function g = eval_g ( x )
  g = zeros ( 1, 2 ) ;


  r = sqrt ( x(1)^2 + x(2)^2 );

  if ( r == 0.0 )
    g(1) = 0.0;
    g(2) = 0.0;
    return
  end

  a = sqrt ( r );
  ar = 0.5 / sqrt ( r );

  b = 1.0 + ( sin ( 50.0 * r^0.2 ) )^2;
  br = 10.0 * sin ( 100.0 * r^0.2 ) * r^(-0.8);

  rx1 = x(1) / r;
  rx2 = x(2) / r;

  g(1) = ( ar * b + a * br ) * rx1;
  g(2) = ( ar * b + a * br ) * rx2;

end

%*****************************************************************************80
function h = eval_h ( x )
  h = zeros ( 2, 2 ) ;
  r = sqrt ( x(1)^2 + x(2)^2 );

  rx1 = x(1) / r;
  rx2 = x(2) / r;

  rx1x1 = x(2)^2 / r^3;
  rx1x2 = - x(1) * x(2) / r^3;
  rx2x1 = - x(1) * x(2) / r^3;
  rx2x2 = x(1)^2 / r^3;
%
%  F = A * B
%  dFdX1 = ( Ar * B + A * Br ) * Rx1
%  d2FdX1dX1 = ( Arr * B + Ar * Br ) * Rx1^2 + ( Ar * B + A * Br ) * Rx1x1
%  etc
%
  a = sqrt ( r );
  ar = 0.5 / sqrt ( r );
  arr = - 0.25 / sqrt ( r^3 );

  b = 1.0 + ( sin ( 50.0 * r^0.2 ) )^2;
  br = 10.0 * sin ( 100.0 * r^0.2 ) * r^(-0.8);
  brr = 200.0 * cos ( 100.0 * r^0.2 ) * r^(-1.6) - 10.0 * sin ( 100.0 * r^0.2 ) * 0.8 * r^(-1.8);

  h(1,1) = ( arr * b + 2.0 * ar * br + a * brr ) * rx1 * rx1 + ( ar * b + a * br ) * rx1x1;
  h(1,2) = ( arr * b + 2.0 * ar * br + a * brr ) * rx1 * rx2 + ( ar * b + a * br ) * rx1x2;
  h(2,1) = ( arr * b + 2.0 * ar * br + a * brr ) * rx2 * rx1 + ( ar * b + a * br ) * rx2x1;
  h(2,2) = ( arr * b + 2.0 * ar * br + a * brr ) * rx2 * rx2 + ( ar * b + a * br ) * rx2x2;

end
