%*****************************************************************************80
%
%% Evaluates the objective function for problem 35.
%
%  Licensing:
%    This code is distributed under the GNU LGPL license.
%
%  Modified:
%    24 December 2011
%
%  Author:
%    Enrico Bertolazzi
%    Dipartimento di Ingegneria Meccanica e Strutturale
%    Universita` degli Studi di Trento
%    Via Mesiano 77, I-38123 Trento, Italy
%    email: enrico.bertolazzi@unitn.it
%
%    based on a code of John Burkardt (http://people.sc.fsu.edu/~jburkardt/)
%
%  Reference:
%    Richard Brent,
%    Algorithms for Minimization with Derivatives,
%    Dover, 2002,
%    ISBN: 0-486-41998-3,
%    LC: QA402.5.B74.
%
function RES = Shubert(what,x)
  global Nf_eval Ng_eval Nh_eval ;
  switch what

  case {'name','title'}
    RES = 'The Shubert Function.' ;

  case {'n','size'}
    RES = 2 ;

  case 'm'
    RES = 1 ;

  case {'init','start'}
    RES = [ 0.5, 1.0 ]' ;

  case 'solution'
    RES = [ 0.0, 0.0 ]';

  case 'f'
    Nf_eval = Nf_eval+1 ;
    RES = eval_f( x ) ;

  case 'Df'
    Ng_eval = Ng_eval+1 ;
    RES = eval_g( x ) ;

  case 'F'
    Ng_eval = Ng_eval+1 ;
    RES = eval_g( x )' ;

  case {'JF', 'DDf'}
    Nh_eval = Nh_eval+1;
    RES = eval_h(x ) ;

  otherwise
    error(['The Shubert Function, bad argument:' what]) ;
  end ;

end

%*****************************************************************************80
function f = eval_f ( x )
  factor1 = 0.0;
  for i = 1 : 5
    y = i;
    factor1 = factor1 + y * cos ( ( y + 1.0 ) * x(1) + y );
  end

  factor2 = 0.0;
  for i = 1 : 5
    y = i;
    factor2 = factor2 + y * cos ( ( y + 1.0 ) * x(2) + y );
  end

  f = factor1 * factor2;
end

%*****************************************************************************80
function g = eval_g ( x )
  g = zeros ( 1, 2 );

  factor1 = 0.0;
  df1dx1  = 0.0;
  for i = 1 : 5
    y = i;
    factor1 = factor1 + y * cos ( ( y + 1.0 ) * x(1) + y );
    df1dx1 = df1dx1 - y * ( y + 1.0 ) * sin ( ( y + 1.0 ) * x(1) + y );
  end

  factor2 = 0.0;
  df2dx2 = 0.0;
  for i = 1 : 5
    y = i;
    factor2 = factor2 + y * cos ( ( y + 1.0 ) * x(2) + y );
    df2dx2 = df2dx2 - y * ( y + 1.0 ) * sin ( ( y + 1.0 ) * x(2) + y );
  end

  g(1) = df1dx1 * factor2;
  g(2) = factor1 * df2dx2;
end

%*****************************************************************************80
function h = eval_h ( x )
  h = zeros ( 2, 2 );
  factor1 = 0.0;
  df1dx1  = 0.0;
  df1dx11 = 0.0;
  for i = 1 : 5
    y = i;
    factor1 = factor1 + y * cos ( ( y + 1.0 ) * x(1) + y );
    df1dx1 = df1dx1 - y * ( y + 1.0 ) * sin ( ( y + 1.0 ) * x(1) + y );
    df1dx11 = df1dx11 - y * ( y + 1.0 )^2 * cos ( ( y + 1.0 ) * x(1) + y );
  end

  factor2 = 0.0;
  df2dx2 = 0.0;
  df2dx22 = 0.0;
  for i = 1 : 5
    y = i;
    factor2 = factor2 + y * cos ( ( y + 1.0 ) * x(2) + y );
    df2dx2 = df2dx2 - y * ( y + 1.0 ) * sin ( ( y + 1.0 ) * x(2) + y );
    df2dx22 = df2dx22 - y * ( y + 1.0 )^2 * cos ( ( y + 1.0 ) * x(2) + y );
  end

  h(1,1) = df1dx11 * factor2;
  h(1,2) = df1dx1 * df2dx2;
  h(2,1) = df1dx1 * df2dx2;
  h(2,2) = factor1 * df2dx22;
end
