%*****************************************************************************80
%
%% Evaluates the objective function for problem 34.
%
%  Licensing:
%    This code is distributed under the GNU LGPL license.
%
%  Modified:
%    24 December 2011
%
%  Author:
%    Enrico Bertolazzi
%    Dipartimento di Ingegneria Meccanica e Strutturale
%    Universita` degli Studi di Trento
%    Via Mesiano 77, I-38123 Trento, Italy
%    email: enrico.bertolazzi@unitn.it
%
%    based on a code of John Burkardt (http://people.sc.fsu.edu/~jburkardt/)
%
%  Reference:
%    Richard Brent,
%    Algorithms for Minimization with Derivatives,
%    Dover, 2002,
%    ISBN: 0-486-41998-3,
%    LC: QA402.5.B74.
%
function RES = SixHumpCamelBack(what,x)
  global Nf_eval Ng_eval Nh_eval ;
  switch what

  case {'name','title'}
    RES = 'The Six-Hump Camel-Back Polynomial.' ;

  case {'n','size'}
    RES = 2 ;

  case 'm'
    RES = 1 ;

  case {'init','start'}
    RES = [ -1.5, 0.5 ]' ;

  case 'solution'
    RES = [ -0.0898,  0.7126 ;
             0.0898, -0.7126 ]';

  case 'f'
    Nf_eval = Nf_eval+1 ;
    RES = eval_f( x ) ;

  case 'Df'
    Ng_eval = Ng_eval+1 ;
    RES = eval_g( x ) ;

  case 'F'
    Ng_eval = Ng_eval+1 ;
    RES = eval_g( x )' ;

  case {'JF', 'DDf'}
    Nh_eval = Nh_eval+1;
    RES = eval_h(x ) ;

  otherwise
    error(['The Six-Hump Camel-Back Polynomial, bad argument:' what]) ;
  end ;

end

%*****************************************************************************80
function f = eval_f ( x )
  f = ( 4.0 - 2.1 * x(1)^2 + x(1)^4 / 3.0 ) * x(1)^2 + x(1) * x(2) + 4.0 * ( x(2)^2 - 1.0 ) * x(2)^2;
end

%*****************************************************************************80
function g = eval_g ( x )
  g = zeros ( 1, 2 );
  g(1) = 8.0 * x(1) - 8.4 * x(1)^3 + 2.0 * x(1)^5 + x(2);
  g(2) = x(1) - 8.0 * x(2) + 16.0 * x(2)^3;
end

%*****************************************************************************80
function h = eval_h ( x )
  h = zeros ( 2, 2 );
  h(1,1) = 8.0 - 25.2 * x(1)^2 + 10.0 * x(1)^4;
  h(1,2) = 1.0;
  h(2,1) = 1.0;
  h(2,2) = -8.0 + 48.0 * x(2)^2;
end
