%*****************************************************************************80
%
%% Evaluates the objective function for problem 13.
%
%  Licensing:
%    This code is distributed under the GNU LGPL license.
%
%  Modified:
%    24 December 2011
%
%  Author:
%    Enrico Bertolazzi
%    Dipartimento di Ingegneria Meccanica e Strutturale
%    Universita` degli Studi di Trento
%    Via Mesiano 77, I-38123 Trento, Italy
%    email: enrico.bertolazzi@unitn.it
%
%    based on a code of John Burkardt (http://people.sc.fsu.edu/~jburkardt/)
%
%  Reference:
%    Richard Brent,
%    Algorithms for Minimization with Derivatives,
%    Dover, 2002,
%    ISBN: 0-486-41998-3,
%    LC: QA402.5.B74.
%
function RES = Trigonometric(what,x)
  global Nf_eval Ng_eval Nh_eval Trigonometric_n ;

  if isempty(Trigonometric_n)
    Trigonometric_n = 2;
  end

  switch what

  case 'setup'
    if x < 1 
      error(['Trigonometric(''setup'', '  x ') argument must be >= 1']) ; 
    end
    Trigonometric_n = x ;

  case {'name','title'}
    RES = 'The trigonometric function.' ;

  case {'n','size'}
    RES = Trigonometric_n ;

  case 'm'
    RES = 1 ;

  case {'init','start'}
    RES = ones(Trigonometric_n,1) / Trigonometric_n ;

  case 'solution'
    RES = zeros(Trigonometric_n,1)  ;

  case 'f'
    Nf_eval = Nf_eval+1 ;
    RES = eval_f( x ) ;

  case 'Df'
    Ng_eval = Ng_eval+1 ;
    RES = eval_g( x ) ;

  case 'F'
    Ng_eval = Ng_eval+1 ;
    RES = eval_g( x )' ;

  case {'JF', 'DDf'}
    Nh_eval = Nh_eval+1;
    RES = eval_h(x ) ;

  otherwise
    error(['The trigonometric function, bad argument:' what]) ;
  end ;

end

%*****************************************************************************80
function f = eval_f ( x )
  global Trigonometric_n ;
  n = Trigonometric_n ;
  s1 = sum ( cos(x) );
  f = 0.0;
  for j = 1 : n
    t =  ( n + j ) - sin ( x(j) ) - s1 - j * cos ( x(j) );
    f = f + t * t;
  end
end

%*****************************************************************************80
function g = eval_g ( x )
  global Trigonometric_n ;
  n = Trigonometric_n ;
  g = zeros ( 1, n );

  s1 = sum ( cos ( x(1:n) ) );

  s2 = 0.0;
  for j = 1 : n
    t =  ( n + j ) - sin ( x(j) ) - s1 - j * cos ( x(j) );
    s2 = s2 + t;
    g(j) = ( j * sin ( x(j) ) - cos ( x(j) ) ) * t;
  end

  for j = 1 : n
    g(j) = 2.0 * ( g(j) + sin ( x(j) ) * s2 );
  end
end

%*****************************************************************************80
function h = eval_h ( x )
  global Trigonometric_n ;
  n = Trigonometric_n ;
  h = zeros ( n, n );

  s1 = sum ( cos ( x(1:n) ) );

  for j = 1 : n
    h(j,j) = sin ( x(j) );
  end

  s2 = 0.0;

  for j = 1 : n

    th = cos ( x(j) );
    t = ( n + j ) - h(j,j) - s1 - j * th;
    s2 = s2 + t;
    for k = 1 : j - 1
      h(j,k) = 2.0 * ( sin ( x(k) ) * ( ( n + j + k ) * h(j,j) - th ) - h(j,j) * cos ( x(k) ) );
    end

    h(j,j) =  ( j * ( j + 2 ) + n ) * h(j,j) * h(j,j) + th * ...
      ( th -  ( 2 * j + 2 ) * h(j,j) ) + t * ( j * th + h(j,j) );

  end

  for j = 1 : n
    h(j,j) = 2.0 * ( h(j,j) + cos ( x(j) ) * s2 );
  end

  for i = 1 : n
    h(i,i+1:n) = h(i+1:n,i);
  end
end
