%*****************************************************************************80
%
%% Evaluates the objective function for problem 6.
%
%  Licensing:
%    This code is distributed under the GNU LGPL license.
%
%  Modified:
%    24 December 2011
%
%  Author:
%    Enrico Bertolazzi
%    Dipartimento di Ingegneria Meccanica e Strutturale
%    Universita` degli Studi di Trento
%    Via Mesiano 77, I-38123 Trento, Italy
%    email: enrico.bertolazzi@unitn.it
%
%    based on a code of John Burkardt (http://people.sc.fsu.edu/~jburkardt/)
%
%  Reference:
%    Richard Brent,
%    Algorithms for Minimization with Derivatives,
%    Dover, 2002,
%    ISBN: 0-486-41998-3,
%    LC: QA402.5.B74.
%
function RES = VariablyDimensioned(what,x)
  global Nf_eval Ng_eval Nh_eval VariablyDimemsioned_n ;

  if isempty(VariablyDimemsioned_n)
    VariablyDimemsioned_n = 2;
  end

  switch what

  case 'setup'
    if x < 1 
      error(['VariablyDimensioned(''setup'', ' x ') argument must be >= 1']) ; 
    end
    VariablyDimemsioned_n = x ;

  case {'name','title'}
    RES = 'The variably dimensioned function.' ;

  case {'n','size'}
    RES = VariablyDimemsioned_n ;

  case 'm'
    RES = 1 ;

  case {'init','start'}
    RES = zeros ( VariablyDimemsioned_n, 1 );
    for i = 1 : VariablyDimemsioned_n
      RES(i) = ( VariablyDimemsioned_n - i ) / VariablyDimemsioned_n;
    end

  case 'solution'
    RES = ones(VariablyDimemsioned_n,1) ;

  case 'f'
    Nf_eval = Nf_eval+1 ;
    RES = eval_f( x ) ;

  case 'Df'
    Ng_eval = Ng_eval+1 ;
    RES = eval_g( x ) ;

  case 'F'
    Ng_eval = Ng_eval+1 ;
    RES = eval_g( x )' ;

  case {'JF', 'DDf'}
    Nh_eval = Nh_eval+1;
    RES = eval_h(x ) ;

  otherwise
    error(['The variably dimensioned function, bad argument:' what]) ;
  end ;

end

%*****************************************************************************80
function f = eval_f ( x )
  global VariablyDimemsioned_n ;
  n = VariablyDimemsioned_n ;
  f1 = 0.0;
  for i = 1 : n
    f1 = f1 + i * ( x(i) - 1.0 );
  end

  f2 = 0.0;
  for i = 1 : n
    f2 = f2 + ( x(i) - 1.0 )^2;
  end

  f = f1 * f1 * ( 1.0 + f1 * f1 ) + f2;
end

%*****************************************************************************80
function g = eval_g ( x )
  global VariablyDimemsioned_n ;
  n = VariablyDimemsioned_n ;
  g = zeros ( 1, n );

  f1 = 0.0;
  for i = 1 : n
    f1 = f1 + i * ( x(i) - 1.0 );
  end

  for i = 1 : n
    df1dxi = i;
    df2dxi = 2.0 * ( x(i) - 1.0 );
    g(i) = ( 2.0 * f1 + 4.0 * f1^3 ) * df1dxi + df2dxi;
  end
end

%*****************************************************************************80
function h = eval_h ( x )
  global VariablyDimemsioned_n ;
  n = VariablyDimemsioned_n ;
  h = zeros ( n, n );

  f1 = 0.0;
  for i = 1 : n
    f1 = f1 + i * ( x(i) - 1.0 );
  end

  for i = 1 : n
    df1dxi = i;
    d2f2dxii = 2.0;
    for j = 1 : n
      df1dxj = j;
      h(i,j) = ( 2.0 + 12.0 * f1 * f1 ) * df1dxi * df1dxj;
    end
    h(i,i) = h(i,i) + d2f2dxii;
  end
end
