%*****************************************************************************80
%
%% Evaluates the objective function for problem 7.
%
%  Licensing:
%    This code is distributed under the GNU LGPL license.
%
%  Modified:
%    24 December 2011
%
%  Author:
%    Enrico Bertolazzi
%    Dipartimento di Ingegneria Meccanica e Strutturale
%    Universita` degli Studi di Trento
%    Via Mesiano 77, I-38123 Trento, Italy
%    email: enrico.bertolazzi@unitn.it
%
%    based on a code of John Burkardt (http://people.sc.fsu.edu/~jburkardt/)
%
%  Reference:
%    Richard Brent,
%    Algorithms for Minimization with Derivatives,
%    Dover, 2002,
%    ISBN: 0-486-41998-3,
%    LC: QA402.5.B74.
%
function RES = Watson(what,x)
  global Nf_eval Ng_eval Nh_eval Watson_n ;

  if isempty(Watson_n)
    Watson_n = 2;
  end

  switch what

  case 'setup'
    if x < 2 
      error(['Watson(''setup'', ' x ') argument must be >= 2']) ; 
    end
    Watson_n = x ;

  case {'name','title'}
    RES = 'The Watson function.' ;

  case {'n','size'}
    RES = Watson_n ;

  case 'm'
    RES = 1 ;

  case {'init','start'}
    RES = zeros ( Watson_n, 1 );

  case 'solution'
    RES = [] ;
    if Watson_n == 6
      RES = [ -0.015725, 1.012435, -0.232992, 1.260430, -1.513729, 0.992996 ]';
    elseif Watson_n == 8
      RES = [ -0.000015, 0.999790, 0.014764, 0.146342, 1.000821, -2.617731, 4.104403, -3.143612, 1.052627 ]';
    end

  case 'f'
    Nf_eval = Nf_eval+1 ;
    RES = eval_f( x ) ;

  case 'Df'
    Ng_eval = Ng_eval+1 ;
    RES = eval_g( x ) ;

  case 'F'
    Ng_eval = Ng_eval+1 ;
    RES = eval_g( x )' ;

  case {'JF', 'DDf'}
    Nh_eval = Nh_eval+1;
    RES = eval_h(x ) ;

  otherwise
    error(['The Watson function, bad argument:' what]) ;
  end ;

end

%*****************************************************************************80
function f = eval_f ( x )
  global Watson_n ;
  n = Watson_n ;
  f = 0.0;
  for i = 1 : 29

    s1 = 0.0;
    d = 1.0;
    for j = 2 : n
      s1 = s1 + ( j - 1 ) * d * x(j);
      d = d * i / 29.0;
    end

    s2 = 0.0;
    d = 1.0;
    for j = 1 : n
      s2 = s2 + d * x(j);
      d = d * i / 29.0;
    end

    f = f + ( s1 - s2 * s2 - 1.0 )^2;

  end

  f = f + x(1) * x(1) + ( x(2) - x(1) * x(1) - 1.0 )^2;
end

%*****************************************************************************80
function g = eval_g ( x )
  global Watson_n ;
  n = Watson_n ;
  g = zeros ( 1, n );

  for i = 1 : 29

    d1 = i / 29.0;
    s1 = 0.0;
    d2 = 1.0;
    for j = 2 : n
      s1 = s1 + ( j - 1 ) * d2 * x(j);
      d2 = d2 * i / 29.0;
    end

    s2 = 0.0;
    d2 = 1.0;
    for j = 1 : n
      s2 = s2 + d2 * x(j);
      d2 = d2 * i / 29.0;
    end

    t = s1 - s2 * s2 - 1.0;
    s3 = 2.0 * s2 * i / 29.0;
    d2 = 2.0 / d1;

    for j = 1 : n
      g(j) = g(j) + d2 * ( ( j - 1 ) - s3 ) * t;
      d2 = d2 * i / 29.0;
    end

  end

  t1 = x(2) - x(1) * x(1) - 1.0;

  g(1) = g(1) + 2.0 * x(1) - 4.0 * x(1) * t1;
  g(2) = g(2) + 2.0 * t1;

end

%*****************************************************************************80
function h = eval_h ( x )
  global Watson_n ;
  n = Watson_n ;
  h = zeros ( n, n );

  for i = 1 : 29

    d1 = i / 29.0;
    d2 = 1.0;
    s1 = 0.0;
    s2 = x(1);
    for j = 2 : n
      s1 = s1 + ( j - 1 ) * d2 * x(j);
      d2 = d1 * d2;
      s2 = s2 + d2 * x(j);
    end
    t = 2.0 * ( s1 - s2 * s2 - 1.0 ) * d1 * d1;
    s3 = 2.0 * d1 * s2;
    d2 = 1.0 / d1;
    for j = 1 : n
      t1 = j - 1 - s3;
      h(j,j) = h(j,j) + 2.0 * ( t1 * t1 - t ) * d2 * d2;
      d3 = 1.0 / d1;
      for k = 1 : j - 1
        h(j,k) = h(j,k) + 2.0 * ( t1 * ( ( k - 1 ) - s3 ) - t ) * d2 * d3;
        d3 = d1 * d3;
      end
      d2 = d1 * d2;
    end

  end

  t3 = x(2) - x(1) * x(1) - 1.0;
  h(1,1) = h(1,1) + 2.0 - 4.0 * ( t3 - 2.0 * x(1) * x(1) );
  h(2,2) = h(2,2) + 2.0;
  h(2,1) = h(2,1) - 4.0 * x(1);

  for i = 1 : n
    h(i,i+1:n) = h(i+1:n,i);
  end
end
