%*****************************************************************************80
%
%% Evaluates the objective function for problem 16.
%
%  Licensing:
%    This code is distributed under the GNU LGPL license.
%
%  Modified:
%    24 December 2011
%
%  Author:
%    Enrico Bertolazzi
%    Dipartimento di Ingegneria Meccanica e Strutturale
%    Universita` degli Studi di Trento
%    Via Mesiano 77, I-38123 Trento, Italy
%    email: enrico.bertolazzi@unitn.it
%
%    based on a code of John Burkardt (http://people.sc.fsu.edu/~jburkardt/)
%
%  Reference:
%
%    Evelyn Beale,
%    On an Iterative Method for Finding a Local Minimum of a Function
%    of More than One Variable,
%    Technical Report 25, 
%    Statistical Techniques Research Group,
%    Princeton University, 1958.
%
%    Richard Brent,
%    Algorithms for Minimization with Derivatives,
%    Dover, 2002,
%    ISBN: 0-486-41998-3,
%    LC: QA402.5.B74.
%
function RES = LSBeale(what,x)
  global NF_eval NJF_eval ;
  switch what

  case {'name','title'}
    RES = 'The Beale Function.' ;

  case {'n','size'}
    RES = 2 ;

  case 'm'
    RES = 3 ;

  case {'init','start'}
    RES = [ 1.0, 1.0 ]' ;

  case 'solution'
    RES = [ 3.0, 0.5 ]' ;

  case 'F'
    NF_eval = NF_eval+1 ;
    RES = eval_F( x ) ;

  case 'JF'
    NJF_eval = NJF_eval+1;
    RES = eval_h(x ) ;

  otherwise
    error(['The Beale Function, bad argument:' what]) ;
  end ;

end

%*****************************************************************************80
function F = eval_F ( x )
  f1 = 1.5   - x(1) * ( 1.0 - x(2)    );
  f2 = 2.25  - x(1) * ( 1.0 - x(2) * x(2) );
  f3 = 2.625 - x(1) * ( 1.0 - x(2) * x(2) * x(2) );
  F = [ f1 ; f2 ; f3 ] ;
end

%*****************************************************************************80
function JF = eval_JF ( x )
  JF = zeros( 3, 2 );
  t2 = (x(2) ^ 2);
  JF(1,1) = -1 + x(2)      ; JF(1,2) = x(1);
  JF(2,1) = -1 + t2        ; JF(2,2) = 2 * x(1) * x(2);
  JF(3,1) = -1 + t2 * x(2) ; JF(3,2) = 3 * x(1) * t2;
end
