%*****************************************************************************80
%
%% Evaluates the objective function for problem 10.
%
%  Licensing:
%    This code is distributed under the GNU LGPL license.
%
%  Modified:
%    24 December 2011
%
%  Author:
%    Enrico Bertolazzi
%    Dipartimento di Ingegneria Meccanica e Strutturale
%    Universita` degli Studi di Trento
%    Via Mesiano 77, I-38123 Trento, Italy
%    email: enrico.bertolazzi@unitn.it
%
%    based on a code of John Burkardt (http://people.sc.fsu.edu/~jburkardt/)
%
%  Reference:
%    Richard Brent,
%    Algorithms for Minimization with Derivatives,
%    Dover, 2002,
%    ISBN: 0-486-41998-3,
%    LC: QA402.5.B74.
%
function RES = LSBrownBadlyScaled(what,x)
  global NF_eval NJF_eval ;

  switch what

  case {'name','title'}
    RES = 'The Brown Badly Scaled Function.' ;

  case {'n','size'}
    RES = 2 ;

  case 'm'
    RES = 3 ;

  case {'init','start'}
    RES = [ 1.0 1.0 ]';

  case 'solution'
    RES = [ 1.0E+06, 2.0E-06 ] ;

  case 'F'
    NF_eval = NF_eval+1 ;
    RES = eval_F( x ) ;

  case 'JF'
    NJF_eval = NJF_eval+1;
    RES = eval_JF(x ) ;

  otherwise
    error(['The Brown Badly Scaled Function, bad argument:' what]) ;
  end ;

end

%*****************************************************************************80
function F = eval_F ( x )
  F = zeros(3,1) ;
  F(1) = x(1) - 1000000.0 ;
  F(2) = x(2) - 0.000002 ;
  F(3) = x(1) * x(2) - 2.0 ;
end

%*****************************************************************************80
function JF = eval_JF ( x )
  JF = zeros ( 3, 2 ) ;
  JF(1,1) = 1    ; JF(1,2) = 0    ;
  JF(2,1) = 0    ; JF(2,2) = 1    ;
  JF(3,1) = x(2) ; JF(3,2) = x(1) ;
end
