%*****************************************************************************80
%
%% Evaluates the objective function for problem 12.
%
%  Licensing:
%    This code is distributed under the GNU LGPL license.
%
%  Modified:
%    24 December 2011
%
%  Author:
%    Enrico Bertolazzi
%    Dipartimento di Ingegneria Meccanica e Strutturale
%    Universita` degli Studi di Trento
%    Via Mesiano 77, I-38123 Trento, Italy
%    email: enrico.bertolazzi@unitn.it
%
%    based on a code of John Burkardt (http://people.sc.fsu.edu/~jburkardt/)
%
%  Reference:
%    Richard Brent,
%    Algorithms for Minimization with Derivatives,
%    Dover, 2002,
%    ISBN: 0-486-41998-3,
%    LC: QA402.5.B74.
%
function RES = LSGulf(what,x)
  global NF_eval NJF_eval ;
  switch what

  case {'name','title'}
    RES = 'The Gulf R&D Function.' ;

  case {'n','size'}
    RES = 3 ;

  case 'm'
    RES = 99 ;

  case {'init','start'}
    RES = [ 40.0, 20.0, 1.20 ]' ;

  case 'solution'
    RES = [ 50.0, 25.0, 1.5 ]' ;

  case 'F'
    NF_eval = NF_eval+1 ;
    RES = eval_F( x ) ;

  case 'JF'
    NJF_eval = NJF_eval+1;
    RES = eval_JF(x ) ;

  otherwise
    error(['Gulf R&D function, bad argument:' what]) ;
  end ;

end

%*****************************************************************************80
function F = eval_F ( x )
  F = zeros ( 99, 1 );
  for i = 1 : 99
    arg = i / 100.0;
    r = abs ( ( - 50.0 * log ( arg ) )^( 2.0 / 3.0 ) + 25.0 - x(2) );
    F(i) = exp ( - r^x(3) / x(1) ) - arg;
  end
end

%*****************************************************************************80
function JF = eval_JF ( x )
  JF = zeros ( 99, 3 );
  for i = 1 : 99
    arg = i / 100.0;
    r   = ( - 50.0 * log ( arg ) )^( 2.0 / 3.0 ) + 25.0 - x(2) ;
    t1  = abs(r) ;
    t2  = t1 ^ x(3);
    t3  = x(1) ^ 2;
    t6  = 1 / x(1);
    t8  = exp(-t2 * t6);
    if r > 0
      t11 = -1 ;
    else
      t11 = +1 ;
    end
    t17 = log(t1);
    JF(i,1) = t2 / t3 * t8;
    JF(i,2) = -t2 * x(3) * t11 / t1 * t6 * t8;
    JF(i,3) = -t2 * t17 * t6 * t8;
  end
end
