%*****************************************************************************80
%
%  Licensing:
%    This code is distributed under the GNU LGPL license.
%
%  Modified:
%    24 December 2011
%
%  Author:
%    Enrico Bertolazzi
%    Dipartimento di Ingegneria Meccanica e Strutturale
%    Universita` degli Studi di Trento
%    Via Mesiano 77, I-38123 Trento, Italy
%    email: enrico.bertolazzi@unitn.it
%
%    based on a code of John Burkardt (http://people.sc.fsu.edu/~jburkardt/)
%
%  Reference:
%
%    Michael Powell,
%    An Iterative Method for Finding Stationary Values of a Function
%    of Several Variables,
%    Computer Journal,
%    Volume 5, 1962, pages 147-151.
%
function RES = LSPowellSingularQuartic( what, x )
  global NF_eval NJF_eval ;

  switch what

  case {'name','title'}
    RES = 'The Powell singular quartic function.' ;

  case {'n','size'}
    RES = 4 ;

  case 'm'
    RES = 4 ;

  case {'init','start'}
    RES = [ 3.0, -1.0, 0.0, 1.0 ]' ;

  case 'solution'
    RES = zeros(4,1) ;

  case 'F'
    NF_eval = NF_eval+1 ;
    RES = eval_F( x ) ;

  case 'JF'
    NJF_eval = NJF_eval+1;
    RES = eval_JF(x ) ;

  otherwise
    error(['The Powell singular quartic function, bad argument:' what]) ;
  end ;

end

%*****************************************************************************80
function F = eval_F ( x )
  f1 = x(1) + 10.0 * x(2) ;
  f2 = x(3) - x(4) ;
  f3 = x(2) - 2.0 * x(3) ;
  f4 = x(1) - x(4);
  f  = [ f1 ; f2 ; f3 ; f4 ] ;
end

%*****************************************************************************80
function JF = eval_JF ( X )
  JF = zeros(4,4) ;
  JF(1,1) = 1 ;
  JF(1,2) = 10 ;
  JF(1,3) = 0 ;
  JF(1,4) = 0 ;

  JF(2,1) = 0 ;
  JF(2,2) = 0 ;
  JF(2,3) = 1 ;
  JF(2,4) = -1 ;

  JF(3,1) = 0 ;
  JF(3,2) = 1 ;
  JF(3,3) = -2 ;
  JF(3,4) = 0 ;

  JF(4,1) = 1 ;
  JF(4,1) = 0 ;
  JF(4,1) = 0 ;
  JF(4,1) = -1 ;
end
