%*****************************************************************************80
%
%% Evaluates the objective function for problem 13.
%
%  Licensing:
%    This code is distributed under the GNU LGPL license.
%
%  Modified:
%    24 December 2011
%
%  Author:
%    Enrico Bertolazzi
%    Dipartimento di Ingegneria Meccanica e Strutturale
%    Universita` degli Studi di Trento
%    Via Mesiano 77, I-38123 Trento, Italy
%    email: enrico.bertolazzi@unitn.it
%
%    based on a code of John Burkardt (http://people.sc.fsu.edu/~jburkardt/)
%
%  Reference:
%    Richard Brent,
%    Algorithms for Minimization with Derivatives,
%    Dover, 2002,
%    ISBN: 0-486-41998-3,
%    LC: QA402.5.B74.
%
function RES = LSTrigonometric(what,x)
  global NF_eval NJF_eval Trigonometric_n ;

  if isempty(Trigonometric_n)
    Trigonometric_n = 2;
  end

  switch what

  case 'setup'
    if x < 1 
      error(['Trigonometric(''setup'', '  x ') argument must be >= 1']) ; 
    end
    Trigonometric_n = x ;

  case {'name','title'}
    RES = 'The trigonometric function.' ;

  case {'n','size'}
    RES = Trigonometric_n ;

  case 'm'
    RES = Trigonometric_n ;

  case {'init','start'}
    RES = ones(Trigonometric_n,1) / Trigonometric_n ;

  case 'solution'
    RES = zeros(Trigonometric_n,1)  ;

  case 'F'
    NF_eval = NF_eval+1 ;
    RES = eval_F( x ) ;

  case 'JF'
    NJF_eval = NJF_eval+1;
    RES = eval_JF(x ) ;

  otherwise
    error(['The trigonometric function, bad argument:' what]) ;
  end ;

end

%*****************************************************************************80
function F = eval_F ( x )
  global Trigonometric_n ;
  F  = zeros(Trigonometric_n,1) ;
  n  = Trigonometric_n ;
  s1 = sum ( cos(x) );
  for j = 1 : n
    F(j) =  ( n + j ) - sin ( x(j) ) - s1 - j * cos ( x(j) );
  end
end

%*****************************************************************************80
function JF = eval_JF ( x )
  global Trigonometric_n ;
  JF = zeros(Trigonometric_n,Trigonometric_n) ;
  n  = Trigonometric_n ;
  s1 = sum ( cos(x) );
  for j = 1 : n
    JF(j,j) = j * sin( x(j) ) - cos( x(j) ) ;
    for i = 1 : n
      JF(j,i) = JF(j,i) + sin( x(i) ) ;
    end
  end
end
