%
% Golden section minimization
%
% Find the minimum of fun(x) starting from the interval [a,b]
% N is the numeber of reduction used by the algorithm
%
% On output the whole sequence of the intervals.
%
function Iab = goldenMinimization(fun,a,b,N)
  tau = (sqrt(5)-1)/2 ;
  lambda = b - tau*(b-a) ;
  mu     = a + tau*(b-a) ;
  fa     = feval(fun,a) ;
  fb     = feval(fun,b) ;
  fl     = feval(fun,lambda) ;
  fm     = feval(fun,mu) ;
  Iab = [a,b,lambda,mu] ;
  for i=2:N
    % check the the interval (left or right)
    if fl < fm
      % select left interval
      b      = mu          ; fb = fm ;
      mu     = lambda      ; fm = fl ;
      lambda = b-tau*(b-a) ; fl = feval(fun,lambda) ;
    else
      % select right interval
      a      = lambda      ; fa = fl ;
      lambda = mu          ; fl = fm ;
      mu     = a+tau*(b-a) ; fm = feval(fun,mu) ;
    end
    Iab = [ Iab ; a, b, lambda, mu ] ;
  end
end
