function [ierr,X,F,NG] = BFGS( fun, X0, varargin )
  ierr = 0 ;
  fX0  = feval( fun, 'f', X0 ) ;
  opts = { 'tol', 'linesearch', 'maxiter', 'f0' };
  defs = {  1E-8,            1,       100, fX0 } ;

  OPTS = parseArgs( opts, defs, varargin{:} ) ;

  TOL        = OPTS.tol ;
  maxiter    = OPTS.maxiter ;
  linesearch = OPTS.linesearch ;
  
  if any(isnan(X0))
    disp(X0) ;
    fprintf( 'GradientMinimization: Bad initial point!\n' ) ;
    ierr = 1 ;
    return ;
  end

  if linesearch < 1 | linesearch > 3
    fprintf( 'GradientMinimization: lineserarch=%d must be in [1,3]\n',linesearch) ;
    ierr = 2 ;
    return ;
  end

  F  = [] ;
  X  = [] ;
  NG = [] ;
  x  = X0 ;
  g  = feval( fun, 'Df',  x )' ; % va trasposto!
  H  = eye( feval( fun, 'size') ) ; % inv(feval( fun, 'DDf', x )) ;
  for k=1:maxiter
    % save values
    f  = feval( fun, 'f',   x )  ;
    X  = [ X x ] ;
    F  = [ F f ] ;
    NG = [ NG norm(g,'inf') ] ;
    % check if finished
    if ( NG(end) < TOL )
      fprintf(1, '\n\nsolution found at iter %d |g|=%g!\n\n',k,NG(end));
      return ;
    end
    % direction search
    d = -H*g ;
    switch linesearch 
      case 1 ; alpha = ArmijoSimpleSearch( fun, x, d, varargin{:} ) ;
      case 2 ; alpha = ArmijoLineSearch( fun, x, d, varargin{:} ) ;
      case 3 ; alpha = WolfeLineSearch( fun, x, d, varargin{:} ) ;
    end
    
    if ~isfinite(alpha)
      ierr = 3 ;
      return ;
    end ;
    
    % update x
    x = x + alpha * d ;
    % update H
    gnew = feval( fun, 'Df', x )' ; % va trasposto!
    y  = gnew - g ;
    z  = H*y ;
    g  = gnew ;
    dy = dot(d,y) ;
    H  = H - (z*d'+d*z')/dy+((alpha+dot(y,z)/dy)/dy)*(d*d') ;
  end
  ierr = 4 ;
end

