function [ierr,X,F,NG] = ConjugateGradientMinimization( fun, X0, varargin )
  ierr = 0 ; % ok found solution
  fX0  = feval( fun, 'f', X0 ) ;
  opts = { 'tol', 'cgtype', 'linesearch', 'maxiter', 'c1', 'c2', 'tau', 'lmin', 'lmax', 'f0' };
  defs = {  1E-8,        1,            1,       100, 1E-3,  0.5,   0.5,   1E-8,   1E+8, fX0 } ;

  OPTS = parseArgs( opts, defs, varargin{:} ) ;

  TOL        = OPTS.tol ;
  tau        = OPTS.tau ;
  c1         = OPTS.c1 ;
  c2         = OPTS.c2 ;
  maxiter    = OPTS.maxiter ;
  linesearch = OPTS.linesearch ;
  cgtype     = OPTS.cgtype ;

  if cgtype < 1 | cgtype > 4
    fprintf( 'ConjugateGradientMinimization: cgtype=%d must be in [1,4]\n',cgtype) ;
    ierr = 1 ;
    return ;
  end
  
  if any(isnan(X0))
    disp(X0) ;
    fprintf( 1, 'ConjugateGradientMinimization: Bad initial point!' ) ;
    ierr = 2 ;
    return ;
  end

  if linesearch < 1 | linesearch > 3
    fprintf( 'ConjugateGradientMinimization: lineserarch=%d must be in [1,3]\n',linesearch) ;
    ierr = 3 ;
    return ;
  end
  
  xk = X0 ;
  F  = [] ;
  X  = [] ;
  NG = [] ;
  for k=1:maxiter
    % evaluate function and gradient
    fk = feval( fun, 'f',  xk ) ;
    Gk = feval( fun, 'Df', xk ) ;
    % save values
    X  = [ X xk ] ;
    F  = [ F fk ] ;
    NG = [ NG norm(Gk,'inf') ] ;
    % check if finished
    if ( NG(end) < TOL )
      return ;
    end
    % direction search
    if k > 1
      beta = 0 ;
      switch cgtype 
        case 1 ;
        case 2 ; beta = (Gk * Gk') / (Gk0 * Gk0') ; % Fletcher and Reeves
        case 3 ; beta = ((Gk-Gk0) * Gk') / (Gk0 * Gk0') ; % Polack and Ribiere
        case 4 ; beta = ((Gk-Gk0) * Gk') / (Gk0 * Gk0') ; % Polack and Ribiere N2
                 beta = max(0,beta) ;
      end
      d = -Gk' + beta * d ;
%      if Gk*d > 0
%        disp('reverse direction!');
%        d = -d ;
%      end
    else
      d = -Gk' ;      
    end
    Gk0 = Gk ; % save old gradient
    switch linesearch 
      case 1 ; alpha = ArmijoSimpleSearch( fun, xk, d, 'c1', c1, 'tau', tau ) ;
      case 2 ; alpha = ArmijoLineSearch( fun, xk, d, 'c1', c1 ) ;
      case 3 ; alpha = WolfeLineSearch( fun, xk, d, 'c1', c1, 'c2', c2 ) ;
    end
    if isnan( alpha )
      ierr = 4 ;
      return ;
    end
    xk = xk + alpha * d ;
  end
  ierr = 5 ;
end

