function TestCoDoDol

global Nf_eval Ng_eval Nh_eval FUNZIONE ;

addpath('../utils') ;
addpath('../functions') ;
addpath('../linesearch') ;
addpath('../minimizer') ;
addpath('../CoDoSol') ;

flist = GetFunctionToMinimize('list') ;
%flist = { @GregoryAndKarney,
%          @Powell3D,
%          @Rosenbrock } ;
%flist = { @DeJongF2 } ;
%flist = { @Rosenbrock } ;

trname = {'tr=Spherical','tr=Elliptical'} ;
scname = {'scalig=Coleman-Li','scaling=Hager'} ;
nok = 0 ;
tot = length(flist) ;
for i=1:tot
  fun      = flist{i} ;
  FUNZIONE = fun ;
  n        = feval(fun, 'n') ;
  x0       = feval(fun, 'start') ;
  tit{i}   = feval(fun, 'title' ) ;
  disp( tit{i} ) ;

  l   = -ones(n,1)*Inf ;
  u   =  ones(n,1)*Inf ;
  ok{i}    = 'failed' ;    
  niter{i} = Inf ;
  nf{i}    = 0 ;
  ng{i}    = 0 ;
  nh{i}    = 0 ;
  nsc{i}   = '' ;
  ntr{i}   = '' ;
  for scaling=0:1
    for tr=0:1
      Nf_eval = 0 ;
      Ng_eval = 0 ;
      Nh_eval = 0 ;
      parms = [1000,Inf,tr,-2,scaling,0] ;
      [sol,ierr,output,history,grad,diagnostic]=CoDoSol(x0,@F,l,u,[1E-8,1E-10],parms,@Fjac) ;
      if output(1) < niter{i} & ierr == 0
        if ~isfinite(niter{i}) 
          nok   = nok + 1 ;
          ok{i} = 'ok' ;
        end
        niter{i} = output(1) ;
        nf{i}    = Nf_eval ;
        ng{i}    = Ng_eval ;
        nh{i}    = Nh_eval ;
        nsc{i}   = scname{scaling+1} ;
        ntr{i}   = trname{tr+1} ;
      end
    end
  end
end

fprintf( 1, ' ok?    iter    nf     ng     nh\n' ) ;
for i=1:length(flist)
  fprintf( 1, '      +------+------+------+------+\n' ) ;
  fprintf( 1, '%6s| %4d | %4d | %4d | %4d | %s %s %s\n',ok{i},niter{i},nf{i},ng{i},nh{i},ntr{i},nsc{i},tit{i} ) ;
end
fprintf( 1, '      +------+------+------+------+\n' ) ;
fprintf( 1, 'ok = %d on a total of %d (%g%%)\n', nok, tot, 100*nok/tot ) ;

end

function RES = F(x)
  global Nf_eval Ng_eval Nh_eval FUNZIONE ;
  RES = feval( FUNZIONE, 'F', x ) ;
end

function RES = Fjac(x)
  global Nf_eval Ng_eval Nh_eval FUNZIONE ;
  RES = feval( FUNZIONE, 'JF', x ) ;
end
