%   Copyright (C) 2011 Enrico Bertolazzi
% 
%   This program is free software; you can redistribute it and/or
%   modify it under the terms of the GNU General Public License
%   as published by the Free Software Foundation; either version 2
%   of the License, or (at your option) any later version.
% 
%   This program is distributed in the hope that it will be useful,
%   but WITHOUT ANY WARRANTY; without even the implied warranty of
%   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%   GNU General Public License for more details. 
% 
%   You should have received a copy of the GNU General Public License
%   along with this program; if not, write to the Free Software
%   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
%   USA.
%
function plotIter( fun, X, F, NG )
  global Nf_eval Ng_eval Nh_eval ;
  niter = length(F) ;
  if feval(fun, 'n') == 2
    xmin = min(X(1,:)) ;
    xmax = max(X(1,:)) ;
    ymin = min(X(2,:)) ;
    ymax = max(X(2,:)) ;
    dx   = xmax-xmin ;
    dy   = ymax-ymin ;
    dd   = max( dx, dy )/2 ;
    xmin = min( X(1,1) - dd, min( X(1,end) - dd, xmin)) ;
    xmax = max( X(1,1) + dd, max( X(1,end) + dd, xmax)) ;
    ymin = min( X(2,1) - dd, min( X(2,end) - dd, ymin)) ;
    ymax = max( X(2,1) + dd, max( X(2,end) + dd, ymax)) ;
    dx   = dx/100 ;
    dy   = dy/100 ;
    xx   = [xmin:dx:xmax] ;
    yy   = [ymin:dy:ymax] ;
    nx   = length(xx) ;
    ny   = length(yy) ;
    zz   = zeros( ny, nx ) ;
    for i=1:nx
      for j=1:ny
        %zz(j,i) = log(1+feval( fun, 'f', [xx(i);yy(j)] )) ;
        zz(j,i) = feval( fun, 'f', [xx(i);yy(j)] ) ;
      end
    end
    zz = sqrt(sqrt(zz - min(min(zz)))) ;

    subplot(3,1,1)
    contourf( xx, yy, zz ) ;
    hold on
    plot( X(1,:), X(2,:),     '-or', 'Linewidth', 1 );
    plot( X(1,1), X(2,1),     'o', 'MarkerFaceColor','r','MarkerSize',10 );
    plot( X(1,end), X(2,end), 'o', 'MarkerFaceColor','g','MarkerSize',10 );
    hold off
    subplot(3,1,2)
    semilogy( (1:niter), F-min(F)+1, '-or', 'Linewidth', 1 );
    subplot(3,1,3)
    semilogy( (1:niter), NG, '-or', 'Linewidth', 1 );
  else
    subplot(2,1,1)
    semilogy( (1:niter), F-min(F)+1, '-or', 'Linewidth', 1 );
    subplot(2,1,2)
    semilogy( (1:niter), NG, '-or', 'Linewidth', 1 );
  end

  fprintf(1, 'niter               %d\n', niter);
  fprintf(1, 'function evaluation %d\n', Nf_eval);
  fprintf(1, 'gradient evaluation %d\n', Ng_eval);
  fprintf(1, 'hessian  evaluation %d\n', Nh_eval);
  fprintf(1, 'last f %d\n', F(end));
  fprintf(1, 'last |gradient| %d\n', NG(end));
  fprintf(1, 'last point %g\n', X(:,end)');
end